@@ -10,6 +10,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import typing as ty
+
 from openstack.baremetal.v1 import _common
 from openstack.baremetal.v1 import allocation as _allocation
 from openstack.baremetal.v1 import chassis as _chassis
@@ -61,10 +63,8 @@ class Proxy(proxy.Proxy):
             kwargs['fields'] = _common.fields_type(fields, resource_type)
         return res.fetch(
             self,
-            error_message="No {resource_type} found for {value}".format(
-                resource_type=resource_type.__name__, value=value
-            ),
-            **kwargs
+            error_message=f"No {resource_type.__name__} found for {value}",
+            **kwargs,
         )
 
     def chassis(self, details=False, **query):
@@ -118,7 +118,7 @@ class Proxy(proxy.Proxy):
 
         :param str name_or_id: The ID of a chassis.
         :param bool ignore_missing: When set to ``False``, an exception of
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the chassis does not exist.  When set to `True``, None will
             be returned when attempting to find a nonexistent chassis.
         :returns: One :class:`~openstack.baremetal.v1.chassis.Chassis` object
@@ -136,7 +136,7 @@ class Proxy(proxy.Proxy):
         :param fields: Limit the resource fields to fetch.
 
         :returns: One :class:`~openstack.baremetal.v1.chassis.Chassis`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             chassis matching the name or ID could be found.
         """
         return self._get_with_fields(_chassis.Chassis, chassis, fields=fields)
@@ -172,7 +172,7 @@ class Proxy(proxy.Proxy):
         :param chassis: The value can be either the ID of a chassis or
             a :class:`~openstack.baremetal.v1.chassis.Chassis` instance.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the chassis could not be found. When set to ``True``, no
             exception will be raised when attempting to delete a non-existent
             chassis.
@@ -206,7 +206,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.baremetal.v1.driver.Driver` instance.
 
         :returns: One :class:`~openstack.baremetal.v1.driver.Driver`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             driver matching the name could be found.
         """
         return self._get(_driver.Driver, driver)
@@ -218,7 +218,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.baremetal.v1.driver.Driver` instance.
 
         :returns: One :dict: of vendor methods with corresponding usages
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             driver matching the name could be found.
         """
         driver = self.get_driver(driver)
@@ -310,7 +310,7 @@ class Proxy(proxy.Proxy):
 
         :param str name_or_id: The name or ID of a node.
         :param bool ignore_missing: When set to ``False``, an exception of
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the node does not exist.  When set to `True``, None will
             be returned when attempting to find a nonexistent node.
         :returns: One :class:`~openstack.baremetal.v1.node.Node` object
@@ -328,7 +328,7 @@ class Proxy(proxy.Proxy):
         :param fields: Limit the resource fields to fetch.
 
         :returns: One :class:`~openstack.baremetal.v1.node.Node`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             node matching the name or ID could be found.
         """
         return self._get_with_fields(_node.Node, node, fields=fields)
@@ -340,7 +340,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.baremetal.v1.node.Node` instance.
 
         :returns: The node inventory
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             inventory could be found.
         """
         res = self._get_resource(_node.Node, node)
@@ -558,9 +558,8 @@ class Proxy(proxy.Proxy):
         try:
             for count in utils.iterate_timeout(
                 timeout,
-                "Timeout waiting for nodes %(nodes)s to reach "
-                "target state '%(state)s'"
-                % {'nodes': log_nodes, 'state': expected_state},
+                f"Timeout waiting for nodes {log_nodes} to reach "
+                f"target state '{expected_state}'",
             ):
                 nodes = [self.get_node(n) for n in remaining]
                 remaining = []
@@ -702,7 +701,7 @@ class Proxy(proxy.Proxy):
         :param node: The value can be either the name or ID of a node or
             a :class:`~openstack.baremetal.v1.node.Node` instance.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the node could not be found. When set to ``True``, no
             exception will be raised when attempting to delete a non-existent
             node.
@@ -773,7 +772,7 @@ class Proxy(proxy.Proxy):
 
         :param str name_or_id: The ID of a port.
         :param bool ignore_missing: When set to ``False``, an exception of
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the port does not exist.  When set to `True``, None will
             be returned when attempting to find a nonexistent port.
         :returns: One :class:`~openstack.baremetal.v1.port.Port` object
@@ -791,7 +790,7 @@ class Proxy(proxy.Proxy):
         :param fields: Limit the resource fields to fetch.
 
         :returns: One :class:`~openstack.baremetal.v1.port.Port`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             port matching the name or ID could be found.
         """
         return self._get_with_fields(_port.Port, port, fields=fields)
@@ -827,7 +826,7 @@ class Proxy(proxy.Proxy):
         :param port: The value can be either the ID of a port or
             a :class:`~openstack.baremetal.v1.port.Port` instance.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the port could not be found. When set to ``True``, no
             exception will be raised when attempting to delete a non-existent
             port.
@@ -892,7 +891,7 @@ class Proxy(proxy.Proxy):
 
         :param str name_or_id: The name or ID of a portgroup.
         :param bool ignore_missing: When set to ``False``, an exception of
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the port group does not exist.  When set to `True``, None will
             be returned when attempting to find a nonexistent port group.
         :returns: One :class:`~openstack.baremetal.v1.port_group.PortGroup`
@@ -910,7 +909,7 @@ class Proxy(proxy.Proxy):
         :param fields: Limit the resource fields to fetch.
 
         :returns: One :class:`~openstack.baremetal.v1.port_group.PortGroup`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             port group matching the name or ID could be found.
         """
         return self._get_with_fields(
@@ -952,7 +951,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.baremetal.v1.port_group.PortGroup`
             instance.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the port group could not be found. When set to ``True``, no
             exception will be raised when attempting to delete a non-existent
             port group.
@@ -964,7 +963,15 @@ class Proxy(proxy.Proxy):
             _portgroup.PortGroup, port_group, ignore_missing=ignore_missing
         )
 
-    def attach_vif_to_node(self, node, vif_id, retry_on_conflict=True):
+    def attach_vif_to_node(
+        self,
+        node: ty.Union[_node.Node, str],
+        vif_id: str,
+        retry_on_conflict: bool = True,
+        *,
+        port_id: ty.Optional[str] = None,
+        port_group_id: ty.Optional[str] = None,
+    ) -> None:
         """Attach a VIF to the node.
 
         The exact form of the VIF ID depends on the network interface used by
@@ -974,17 +981,29 @@ class Proxy(proxy.Proxy):
 
         :param node: The value can be either the name or ID of a node or
             a :class:`~openstack.baremetal.v1.node.Node` instance.
-        :param string vif_id: Backend-specific VIF ID.
+        :param vif_id: Backend-specific VIF ID.
         :param retry_on_conflict: Whether to retry HTTP CONFLICT errors.
             This can happen when either the VIF is already used on a node or
             the node is locked. Since the latter happens more often, the
             default value is True.
-        :return: ``None``
+        :param port_id: The UUID of the port to attach the VIF to. Only one of
+            port_id or port_group_id can be provided.
+        :param port_group_id: The UUID of the portgroup to attach to. Only one
+            of port_group_id or port_id can be provided.
+        :return: None
         :raises: :exc:`~openstack.exceptions.NotSupported` if the server
             does not support the VIF API.
+        :raises: :exc:`~openstack.exceptions.InvalidRequest` if both port_id
+            and port_group_id are provided.
         """
         res = self._get_resource(_node.Node, node)
-        res.attach_vif(self, vif_id, retry_on_conflict=retry_on_conflict)
+        res.attach_vif(
+            self,
+            vif_id=vif_id,
+            retry_on_conflict=retry_on_conflict,
+            port_id=port_id,
+            port_group_id=port_group_id,
+        )
 
     def detach_vif_from_node(self, node, vif_id, ignore_missing=True):
         """Detach a VIF from the node.
@@ -997,7 +1016,7 @@ class Proxy(proxy.Proxy):
             a :class:`~openstack.baremetal.v1.node.Node` instance.
         :param string vif_id: Backend-specific VIF ID.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the VIF does not exist. Otherwise, ``False``
             is returned.
         :return: ``True`` if the VIF was detached, otherwise ``False``.
@@ -1075,7 +1094,7 @@ class Proxy(proxy.Proxy):
         :param fields: Limit the resource fields to fetch.
 
         :returns: One :class:`~openstack.baremetal.v1.allocation.Allocation`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             allocation matching the name or ID could be found.
         """
         return self._get_with_fields(
@@ -1115,7 +1134,7 @@ class Proxy(proxy.Proxy):
         :param allocation: The value can be the name or ID of an allocation or
             a :class:`~openstack.baremetal.v1.allocation.Allocation` instance.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the allocation could not be found. When set to ``True``, no
             exception will be raised when attempting to delete a non-existent
             allocation.
@@ -1167,7 +1186,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.baremetal.v1.node.Node` instance.
         :param trait: trait to remove from the node.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the trait could not be found. When set to ``True``, no
             exception will be raised when attempting to delete a non-existent
             trait.
@@ -1243,6 +1262,16 @@ class Proxy(proxy.Proxy):
         res = self._get_resource(_node.Node, node)
         return res.set_traits(self, traits)
 
+    def list_node_firmware(self, node):
+        """Lists firmware components for a node.
+
+        :param node: The value can be the name or ID of a node or a
+            :class:`~openstack.baremetal.v1.node.Node` instance.
+        :returns: A list of the node's firmware components.
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.list_firmware(self)
+
     def volume_connectors(self, details=False, **query):
         """Retrieve a generator of volume_connector.
 
@@ -1300,7 +1329,7 @@ class Proxy(proxy.Proxy):
         :param str vc_id: The ID of a volume connector.
 
         :param bool ignore_missing: When set to ``False``, an exception of
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the volume connector does not exist.  When set to `True``,
             None will be returned when attempting to find a nonexistent
             volume connector.
@@ -1325,7 +1354,7 @@ class Proxy(proxy.Proxy):
 
         :returns: One
             :class: `~openstack.baremetal.v1.volume_connector.VolumeConnector`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             volume_connector matching the name or ID could be found.`
         """
         return self._get_with_fields(
@@ -1375,7 +1404,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.baremetal.v1.volume_connector.VolumeConnector`
             instance.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the volume_connector could not be found.
             When set to ``True``, no exception will be raised when
             attempting to delete a non-existent volume_connector.
@@ -1447,7 +1476,7 @@ class Proxy(proxy.Proxy):
         :param str vt_id: The ID of a volume target.
 
         :param bool ignore_missing: When set to ``False``, an exception of
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the volume connector does not exist.  When set to `True``,
             None will be returned when attempting to find a nonexistent
             volume target.
@@ -1470,7 +1499,7 @@ class Proxy(proxy.Proxy):
 
         :returns: One
             :class:`~openstack.baremetal.v1.volume_target.VolumeTarget`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             volume_target matching the name or ID could be found.`
         """
         return self._get_with_fields(
@@ -1517,7 +1546,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.baremetal.v1.volume_target.VolumeTarget`
             instance.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the volume_target could not be found.
             When set to ``True``, no exception will be raised when
             attempting to delete a non-existent volume_target.
@@ -1585,7 +1614,7 @@ class Proxy(proxy.Proxy):
             instance.
 
         :param bool ignore_missing: When set to ``False``,
-            an exception:class:`~openstack.exceptions.ResourceNotFound`
+            an exception:class:`~openstack.exceptions.NotFoundException`
             will be raised when the deploy_template
             could not be found.
             When set to ``True``, no
@@ -1616,7 +1645,7 @@ class Proxy(proxy.Proxy):
 
         :returns: One
             :class:`~openstack.baremetal.v1.deploy_templates.DeployTemplate`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no deployment template matching the name or
             ID could be found.
         """
@@ -1663,7 +1692,7 @@ class Proxy(proxy.Proxy):
 
         :returns: One :class:`~openstack.baremetal.v1.conductor.Conductor`
 
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             conductor matching the name could be found.
         """
         return self._get_with_fields(
