@@ -12,11 +12,14 @@
 
 import collections
 import enum
+import typing as ty
+import warnings
 
 from openstack.baremetal.v1 import _common
 from openstack import exceptions
 from openstack import resource
 from openstack import utils
+from openstack import warnings as os_warnings
 
 
 class ValidationResult:
@@ -99,8 +102,8 @@ class Node(_common.Resource):
         is_maintenance='maintenance',
     )
 
-    # Ability to have a firmware_interface on a node.
-    _max_microversion = '1.87'
+    # Ability to run predefined sets of steps on a node using runbooks.
+    _max_microversion = '1.92'
 
     # Properties
     #: The UUID of the allocation associated with this node. Added in API
@@ -206,9 +209,13 @@ class Node(_common.Resource):
     #: A string to be used by external schedulers to identify this node as a
     #: unit of a specific type of resource. Added in API microversion 1.21.
     resource_class = resource.Body("resource_class")
-    #: A string represents the current service step being executed upon.
+    #: A string representing the current service step being executed upon.
     #: Added in API microversion 1.87.
     service_step = resource.Body("service_step")
+    #: A string representing the uuid or logical name of a runbook as an
+    #: alternative to providing ``clean_steps`` or ``service_steps``.
+    #: Added in API microversion 1.92.
+    runbook = resource.Body("runbook")
     #: A string indicating the shard this node belongs to. Added in API
     #: microversion 1,82.
     shard = resource.Body("shard")
@@ -275,7 +282,7 @@ class Node(_common.Resource):
             # API version 1.1 uses None instead of "available". Make it
             # consistent.
             attrs['provision_state'] = 'available'
-        return super(Node, self)._consume_body_attrs(attrs)
+        return super()._consume_body_attrs(attrs)
 
     def create(self, session, *args, **kwargs):
         """Create a remote resource based on this instance.
@@ -317,9 +324,8 @@ class Node(_common.Resource):
                 microversion = _common.STATE_VERSIONS[expected_provision_state]
             except KeyError:
                 raise ValueError(
-                    "Node's provision_state must be one of %s for creation, "
-                    "got %s"
-                    % (
+                    "Node's provision_state must be one of {} for creation, "
+                    "got {}".format(
                         ', '.join(_common.STATE_VERSIONS),
                         expected_provision_state,
                     )
@@ -327,7 +333,7 @@ class Node(_common.Resource):
             else:
                 error_message = (
                     "Cannot create a node with initial provision "
-                    "state %s" % expected_provision_state
+                    f"state {expected_provision_state}"
                 )
                 # Nodes cannot be created as available using new API versions
                 maximum = (
@@ -346,9 +352,7 @@ class Node(_common.Resource):
         # Ironic cannot set provision_state itself, so marking it as unchanged
         self._clean_body_attrs({'provision_state'})
 
-        super(Node, self).create(
-            session, *args, microversion=microversion, **kwargs
-        )
+        super().create(session, *args, microversion=microversion, **kwargs)
 
         if (
             expected_provision_state == 'manageable'
@@ -395,7 +399,7 @@ class Node(_common.Resource):
                 # the new status.
                 return self.fetch(session)
 
-        return super(Node, self).commit(session, *args, **kwargs)
+        return super().commit(session, *args, **kwargs)
 
     def set_provision_state(
         self,
@@ -408,6 +412,7 @@ class Node(_common.Resource):
         timeout=None,
         deploy_steps=None,
         service_steps=None,
+        runbook=None,
     ):
         """Run an action modifying this node's provision state.
 
@@ -432,6 +437,7 @@ class Node(_common.Resource):
             and ``rebuild`` target.
         :param service_steps: Service steps to execute, only valid for
             ``service`` target.
+        :param ``runbook``: UUID or logical name of a runbook.
 
         :return: This :class:`Node` instance.
         :raises: ValueError if ``config_drive``, ``clean_steps``,
@@ -461,12 +467,45 @@ class Node(_common.Resource):
         version = self._assert_microversion_for(session, 'commit', version)
 
         body = {'target': target}
+        if runbook:
+            version = self._assert_microversion_for(
+                session, 'commit', _common.RUNBOOKS_VERSION
+            )
+
+            if clean_steps is not None:
+                raise ValueError(
+                    'Please provide either clean steps or a '
+                    'runbook, but not both.'
+                )
+            if service_steps is not None:
+                raise ValueError(
+                    'Please provide either service steps or a '
+                    'runbook, but not both.'
+                )
+
+            if target != 'clean' and target != 'service':
+                msg = (
+                    'A runbook can only be provided when setting target '
+                    'provision state to any of "[clean, service]"'
+                )
+                raise ValueError(msg)
+
+            body['runbook'] = runbook
+
         if config_drive:
             if target not in ('active', 'rebuild'):
                 raise ValueError(
                     'Config drive can only be provided with '
                     '"active" and "rebuild" targets'
                 )
+            if isinstance(config_drive, bytes):
+                try:
+                    config_drive = config_drive.decode('utf-8')
+                except UnicodeError:
+                    raise ValueError(
+                        'Config drive must be a dictionary or a base64 '
+                        'encoded string'
+                    )
             # Not a typo - ironic accepts "configdrive" (without underscore)
             body['configdrive'] = config_drive
 
@@ -506,8 +545,8 @@ class Node(_common.Resource):
                 expected_state = _common.EXPECTED_STATES[target]
             except KeyError:
                 raise ValueError(
-                    'For target %s the expected state is not '
-                    'known, cannot wait for it' % target
+                    f'For target {target} the expected state is not '
+                    'known, cannot wait for it'
                 )
 
         request = self._prepare_request(requires_id=True)
@@ -521,8 +560,8 @@ class Node(_common.Resource):
         )
 
         msg = (
-            "Failed to set provision state for bare metal node {node} "
-            "to {target}".format(node=self.id, target=target)
+            f"Failed to set provision state for bare metal node {self.id} "
+            f"to {target}"
         )
         exceptions.raise_from_response(response, error_message=msg)
 
@@ -548,9 +587,8 @@ class Node(_common.Resource):
         """
         for count in utils.iterate_timeout(
             timeout,
-            "Timeout waiting for node %(node)s to reach "
-            "power state '%(state)s'"
-            % {'node': self.id, 'state': expected_state},
+            f"Timeout waiting for node {self.id} to reach "
+            f"power state '{expected_state}'",
         ):
             self.fetch(session)
             if self.power_state == expected_state:
@@ -589,9 +627,8 @@ class Node(_common.Resource):
         """
         for count in utils.iterate_timeout(
             timeout,
-            "Timeout waiting for node %(node)s to reach "
-            "target state '%(state)s'"
-            % {'node': self.id, 'state': expected_state},
+            f"Timeout waiting for node {self.id} to reach "
+            f"target state '{expected_state}'",
         ):
             self.fetch(session)
             if self._check_state_reached(
@@ -637,7 +674,7 @@ class Node(_common.Resource):
 
         for count in utils.iterate_timeout(
             timeout,
-            "Timeout waiting for the lock to be released on node %s" % self.id,
+            f"Timeout waiting for the lock to be released on node {self.id}",
         ):
             self.fetch(session)
             if self.reservation is None:
@@ -679,13 +716,8 @@ class Node(_common.Resource):
             or self.provision_state == 'error'
         ):
             raise exceptions.ResourceFailure(
-                "Node %(node)s reached failure state \"%(state)s\"; "
-                "the last error is %(error)s"
-                % {
-                    'node': self.id,
-                    'state': self.provision_state,
-                    'error': self.last_error,
-                }
+                f"Node {self.id} reached failure state \"{self.provision_state}\"; "
+                f"the last error is {self.last_error}"
             )
         # Special case: a failure state for "manage" transition can be
         # "enroll"
@@ -695,10 +727,9 @@ class Node(_common.Resource):
             and self.last_error
         ):
             raise exceptions.ResourceFailure(
-                "Node %(node)s could not reach state manageable: "
+                f"Node {self.id} could not reach state manageable: "
                 "failed to verify management credentials; "
-                "the last error is %(error)s"
-                % {'node': self.id, 'error': self.last_error}
+                f"the last error is {self.last_error}"
             )
 
     def inject_nmi(self, session):
@@ -724,7 +755,7 @@ class Node(_common.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to inject NMI to node {node}".format(node=self.id)
+        msg = f"Failed to inject NMI to node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
     def set_power_state(self, session, target, wait=False, timeout=None):
@@ -749,8 +780,8 @@ class Node(_common.Resource):
                 expected = _common.EXPECTED_POWER_STATES[target]
             except KeyError:
                 raise ValueError(
-                    "Cannot use target power state %s with wait, "
-                    "the expected state is not known" % target
+                    f"Cannot use target power state {target} with wait, "
+                    "the expected state is not known"
                 )
 
         session = self._get_session(session)
@@ -776,15 +807,23 @@ class Node(_common.Resource):
         )
 
         msg = (
-            "Failed to set power state for bare metal node {node} "
-            "to {target}".format(node=self.id, target=target)
+            f"Failed to set power state for bare metal node {self.id} "
+            f"to {target}"
         )
         exceptions.raise_from_response(response, error_message=msg)
 
         if wait:
             self.wait_for_power_state(session, expected, timeout=timeout)
 
-    def attach_vif(self, session, vif_id, retry_on_conflict=True):
+    def attach_vif(
+        self,
+        session,
+        vif_id: str,
+        retry_on_conflict: bool = True,
+        *,
+        port_id: ty.Optional[str] = None,
+        port_group_id: ty.Optional[str] = None,
+    ) -> None:
         """Attach a VIF to the node.
 
         The exact form of the VIF ID depends on the network interface used by
@@ -794,26 +833,46 @@ class Node(_common.Resource):
 
         :param session: The session to use for making this request.
         :type session: :class:`~keystoneauth1.adapter.Adapter`
-        :param string vif_id: Backend-specific VIF ID.
+        :param vif_id: Backend-specific VIF ID.
         :param retry_on_conflict: Whether to retry HTTP CONFLICT errors.
             This can happen when either the VIF is already used on a node or
             the node is locked. Since the latter happens more often, the
             default value is True.
-        :return: ``None``
+        :param port_id: The UUID of the port to attach the VIF to. Only one of
+            port_id or port_group_id can be provided.
+        :param port_group_id: The UUID of the portgroup to attach to. Only one
+            of port_group_id or port_id can be provided.
+        :return: None
         :raises: :exc:`~openstack.exceptions.NotSupported` if the server
             does not support the VIF API.
+        :raises: :exc:`~openstack.exceptions.InvalidRequest` if both port_id
+            and port_group_id are provided.
         """
+        if port_id and port_group_id:
+            msg = (
+                'Only one of vif_port_id and vif_portgroup_id can be provided'
+            )
+            raise exceptions.InvalidRequest(msg)
+
         session = self._get_session(session)
+        if port_id or port_group_id:
+            required_version = _common.VIF_OPTIONAL_PARAMS_VERSION
+        else:
+            required_version = _common.VIF_VERSION
         version = self._assert_microversion_for(
             session,
             'commit',
-            _common.VIF_VERSION,
+            required_version,
             error_message=("Cannot use VIF attachment API"),
         )
 
         request = self._prepare_request(requires_id=True)
         request.url = utils.urljoin(request.url, 'vifs')
         body = {'id': vif_id}
+        if port_id:
+            body['port_uuid'] = port_id
+        elif port_group_id:
+            body['portgroup_uuid'] = port_group_id
         retriable_status_codes = _common.RETRIABLE_STATUS_CODES
         if not retry_on_conflict:
             retriable_status_codes = list(set(retriable_status_codes) - {409})
@@ -825,9 +884,7 @@ class Node(_common.Resource):
             retriable_status_codes=retriable_status_codes,
         )
 
-        msg = "Failed to attach VIF {vif} to bare metal node {node}".format(
-            node=self.id, vif=vif_id
-        )
+        msg = f"Failed to attach VIF {vif_id} to bare metal node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
     def detach_vif(self, session, vif_id, ignore_missing=True):
@@ -841,7 +898,7 @@ class Node(_common.Resource):
         :type session: :class:`~keystoneauth1.adapter.Adapter`
         :param string vif_id: Backend-specific VIF ID.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    :class:`~openstack.exceptions.NotFoundException` will be
                     raised when the VIF does not exist. Otherwise, ``False``
                     is returned.
         :return: ``True`` if the VIF was detached, otherwise ``False``.
@@ -872,9 +929,7 @@ class Node(_common.Resource):
             )
             return False
 
-        msg = "Failed to detach VIF {vif} from bare metal node {node}".format(
-            node=self.id, vif=vif_id
-        )
+        msg = f"Failed to detach VIF {vif_id} from bare metal node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
         return True
 
@@ -905,9 +960,7 @@ class Node(_common.Resource):
             request.url, headers=request.headers, microversion=version
         )
 
-        msg = "Failed to list VIFs attached to bare metal node {node}".format(
-            node=self.id
-        )
+        msg = f"Failed to list VIFs attached to bare metal node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
         return [vif['id'] for vif in response.json()['vifs']]
 
@@ -934,21 +987,21 @@ class Node(_common.Resource):
             request.url, headers=request.headers, microversion=version
         )
 
-        msg = "Failed to validate node {node}".format(node=self.id)
+        msg = f"Failed to validate node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
         result = response.json()
 
         if required:
             failed = [
-                '%s (%s)' % (key, value.get('reason', 'no reason'))
+                '{} ({})'.format(key, value.get('reason', 'no reason'))
                 for key, value in result.items()
                 if key in required and not value.get('result')
             ]
 
             if failed:
                 raise exceptions.ValidationException(
-                    'Validation failed for required interfaces of node {node}:'
-                    ' {failures}'.format(
+                    'Validation failed for required interfaces of node '
+                    '{node}: {failures}'.format(
                         node=self.id, failures=', '.join(failed)
                     )
                 )
@@ -990,9 +1043,7 @@ class Node(_common.Resource):
             headers=request.headers,
             microversion=version,
         )
-        msg = "Failed to change maintenance mode for node {node}".format(
-            node=self.id
-        )
+        msg = f"Failed to change maintenance mode for node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
     def get_boot_device(self, session):
@@ -1013,9 +1064,7 @@ class Node(_common.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to get boot device for node {node}".format(
-            node=self.id,
-        )
+        msg = f"Failed to get boot device for node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
         return response.json()
@@ -1044,7 +1093,7 @@ class Node(_common.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to set boot device for node {node}".format(node=self.id)
+        msg = f"Failed to set boot device for node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
     def get_supported_boot_devices(self, session):
@@ -1070,9 +1119,7 @@ class Node(_common.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to get supported boot devices for node {node}".format(
-            node=self.id,
-        )
+        msg = f"Failed to get supported boot devices for node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
         return response.json()
@@ -1096,8 +1143,8 @@ class Node(_common.Resource):
         request.url = utils.urljoin(request.url, 'states', 'boot_mode')
         if target not in ('uefi', 'bios'):
             raise ValueError(
-                "Unrecognized boot mode %s."
-                "Boot mode should be one of 'uefi' or 'bios'." % target
+                f"Unrecognized boot mode {target}."
+                "Boot mode should be one of 'uefi' or 'bios'."
             )
         body = {'target': target}
 
@@ -1109,7 +1156,7 @@ class Node(_common.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to change boot mode for node {node}".format(node=self.id)
+        msg = f"Failed to change boot mode for node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
     def set_secure_boot(self, session, target):
@@ -1132,8 +1179,8 @@ class Node(_common.Resource):
         request.url = utils.urljoin(request.url, 'states', 'secure_boot')
         if not isinstance(target, bool):
             raise ValueError(
-                "Invalid target %s. It should be True or False "
-                "corresponding to secure boot state 'on' or 'off'" % target
+                f"Invalid target {target}. It should be True or False "
+                "corresponding to secure boot state 'on' or 'off'"
             )
         body = {'target': target}
 
@@ -1145,9 +1192,7 @@ class Node(_common.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to change secure boot state for {node}".format(
-            node=self.id
-        )
+        msg = f"Failed to change secure boot state for {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
     def add_trait(self, session, trait):
@@ -1169,9 +1214,7 @@ class Node(_common.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to add trait {trait} for node {node}".format(
-            trait=trait, node=self.id
-        )
+        msg = f"Failed to add trait {trait} for node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
         self.traits = list(set(self.traits or ()) | {trait})
@@ -1182,7 +1225,7 @@ class Node(_common.Resource):
         :param session: The session to use for making this request.
         :param trait: The trait to remove from the node.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the trait does not exist.
             Otherwise, ``False`` is returned.
         :returns bool: True on success removing the trait.
@@ -1243,7 +1286,7 @@ class Node(_common.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to set traits for node {node}".format(node=self.id)
+        msg = f"Failed to set traits for node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
         self.traits = traits
@@ -1261,7 +1304,7 @@ class Node(_common.Resource):
         version = self._get_microversion(session, action='commit')
         request = self._prepare_request(requires_id=True)
         request.url = utils.urljoin(
-            request.url, 'vendor_passthru?method={}'.format(method)
+            request.url, f'vendor_passthru?method={method}'
         )
 
         call = getattr(session, verb.lower())
@@ -1274,10 +1317,8 @@ class Node(_common.Resource):
         )
 
         msg = (
-            "Failed to call vendor_passthru for node {node}, verb {verb}"
-            " and method {method}".format(
-                node=self.id, verb=verb, method=method
-            )
+            f"Failed to call vendor_passthru for node {self.id}, verb {verb} "
+            f"and method {method}"
         )
         exceptions.raise_from_response(response, error_message=msg)
 
@@ -1301,9 +1342,7 @@ class Node(_common.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to list vendor_passthru methods for node {node}".format(
-            node=self.id
-        )
+        msg = f"Failed to list vendor_passthru methods for node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
         return response.json()
@@ -1326,9 +1365,7 @@ class Node(_common.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to get console for node {node}".format(
-            node=self.id,
-        )
+        msg = f"Failed to get console for node {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
         return response.json()
@@ -1346,8 +1383,8 @@ class Node(_common.Resource):
         request.url = utils.urljoin(request.url, 'states', 'console')
         if not isinstance(enabled, bool):
             raise ValueError(
-                "Invalid enabled %s. It should be True or False "
-                "corresponding to console enabled or disabled" % enabled
+                f"Invalid enabled {enabled}. It should be True or False "
+                "corresponding to console enabled or disabled"
             )
         body = {'enabled': enabled}
 
@@ -1359,20 +1396,22 @@ class Node(_common.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to change console mode for {node}".format(
-            node=self.id,
-        )
+        msg = f"Failed to change console mode for {self.id}"
         exceptions.raise_from_response(response, error_message=msg)
 
-    # TODO(stephenfin): Drop 'node_id' and use 'self.id' instead or convert to
-    # a classmethod
     def get_node_inventory(self, session, node_id):
         """Get a node's inventory.
 
         :param session: The session to use for making this request.
-        :param node_id: The ID of the node.
+        :param node_id: **DEPRECATED** The ID of the node.
         :returns: The HTTP response.
         """
+        if node_id is not None:
+            warnings.warn(
+                "The 'node_id' field is unnecessary and will be removed in "
+                "a future release.",
+                os_warnings.RemovedInSDK60Warning,
+            )
         session = self._get_session(session)
         version = self._get_microversion(session, action='fetch')
         request = self._prepare_request(requires_id=True)
@@ -1385,12 +1424,39 @@ class Node(_common.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed to get inventory for node {node}".format(
-            node=node_id,
-        )
+        msg = f"Failed to get inventory for node {node_id}"
         exceptions.raise_from_response(response, error_message=msg)
         return response.json()
 
+    def list_firmware(self, session):
+        """List firmware components associated with the node.
+
+        :param session: The session to use for making this request.
+        :returns: The HTTP response.
+        """
+        session = self._get_session(session)
+        version = self._assert_microversion_for(
+            session,
+            'fetch',
+            _common.FIRMWARE_VERSION,
+            error_message=("Cannot use node list firmware components API"),
+        )
+
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'firmware')
+
+        response = session.get(
+            request.url,
+            headers=request.headers,
+            microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
+        )
+
+        msg = f"Failed to list firmware components for node {self.id}"
+        exceptions.raise_from_response(response, error_message=msg)
+
+        return response.json()
+
     def patch(
         self,
         session,
@@ -1399,6 +1465,8 @@ class Node(_common.Resource):
         has_body=True,
         retry_on_conflict=None,
         base_path=None,
+        *,
+        microversion=None,
         reset_interfaces=None,
     ):
         if reset_interfaces is not None:
@@ -1439,7 +1507,7 @@ class Node(_common.Resource):
             )
 
         else:
-            return super(Node, self).patch(
+            return super().patch(
                 session, patch=patch, retry_on_conflict=retry_on_conflict
             )
 
