@@ -16,7 +16,7 @@ import warnings
 
 import jsonpatch
 
-from openstack.baremetal.v1._proxy import Proxy
+from openstack.cloud import openstackcloud
 from openstack import exceptions
 from openstack import warnings as os_warnings
 
@@ -35,15 +35,13 @@ def _normalize_port_list(nics):
             except KeyError:
                 raise TypeError(
                     "Either 'address' or 'mac' must be provided "
-                    "for port %s" % row
+                    f"for port {row}"
                 )
         ports.append(dict(row, address=address))
     return ports
 
 
-class BaremetalCloudMixin:
-    baremetal: Proxy
-
+class BaremetalCloudMixin(openstackcloud._OpenStackCloudMixin):
     def list_nics(self):
         """Return a list of all bare metal ports."""
         return list(self.baremetal.ports(details=True))
@@ -84,10 +82,7 @@ class BaremetalCloudMixin:
         :rtype: :class:`~openstack.baremetal.v1.node.Node`.
         :returns: The node found or None if no nodes are found.
         """
-        try:
-            return self.baremetal.find_node(name_or_id, ignore_missing=False)
-        except exceptions.NotFoundException:
-            return None
+        return self.baremetal.find_node(name_or_id, ignore_missing=True)
 
     def get_machine_by_mac(self, mac):
         """Get machine by port MAC address
@@ -131,10 +126,9 @@ class BaremetalCloudMixin:
         if node.provision_state == 'available':
             if node.instance_id:
                 raise exceptions.SDKException(
-                    "Refusing to inspect available machine %(node)s "
+                    f"Refusing to inspect available machine {node.id} "
                     "which is associated with an instance "
-                    "(instance_uuid %(inst)s)"
-                    % {'node': node.id, 'inst': node.instance_id}
+                    f"(instance_uuid {node.instance_id})"
                 )
 
             return_to_available = True
@@ -147,10 +141,9 @@ class BaremetalCloudMixin:
 
         if node.provision_state not in ('manageable', 'inspect failed'):
             raise exceptions.SDKException(
-                "Machine %(node)s must be in 'manageable', 'inspect failed' "
+                f"Machine {node.id} must be in 'manageable', 'inspect failed' "
                 "or 'available' provision state to start inspection, the "
-                "current state is %(state)s"
-                % {'node': node.id, 'state': node.provision_state}
+                f"current state is {node.provision_state}"
             )
 
         node = self.baremetal.set_node_provision_state(
@@ -188,7 +181,7 @@ class BaremetalCloudMixin:
         timeout=3600,
         lock_timeout=600,
         provision_state='available',
-        **kwargs
+        **kwargs,
     ):
         """Register Baremetal with Ironic
 
@@ -209,10 +202,7 @@ class BaremetalCloudMixin:
 
             Example::
 
-                [
-                    {'address': 'aa:bb:cc:dd:ee:01'},
-                    {'address': 'aa:bb:cc:dd:ee:02'}
-                ]
+                [{'address': 'aa:bb:cc:dd:ee:01'}, {'address': 'aa:bb:cc:dd:ee:02'}]
 
             Alternatively, you can provide an array of MAC addresses.
         :param wait: Boolean value, defaulting to false, to wait for the node
@@ -237,7 +227,7 @@ class BaremetalCloudMixin:
         if provision_state not in ('enroll', 'manageable', 'available'):
             raise ValueError(
                 'Initial provision state must be enroll, '
-                'manageable or available, got %s' % provision_state
+                f'manageable or available, got {provision_state}'
             )
 
         # Available is tricky: it cannot be directly requested on newer API
@@ -307,15 +297,15 @@ class BaremetalCloudMixin:
         if wait is not None:
             warnings.warn(
                 "wait argument is deprecated and has no effect",
-                os_warnings.OpenStackDeprecationWarning,
+                os_warnings.RemovedInSDK50Warning,
             )
 
         machine = self.get_machine(uuid)
         invalid_states = ['active', 'cleaning', 'clean wait', 'clean failed']
         if machine['provision_state'] in invalid_states:
             raise exceptions.SDKException(
-                "Error unregistering node '%s' due to current provision "
-                "state '%s'" % (uuid, machine['provision_state'])
+                "Error unregistering node '{}' due to current provision "
+                "state '{}'".format(uuid, machine['provision_state'])
             )
 
         # NOTE(TheJulia) There is a high possibility of a lock being present
@@ -326,8 +316,8 @@ class BaremetalCloudMixin:
             self.baremetal.wait_for_node_reservation(machine, timeout)
         except exceptions.SDKException as e:
             raise exceptions.SDKException(
-                "Error unregistering node '%s': Exception occured while"
-                " waiting to be able to proceed: %s" % (machine['uuid'], e)
+                "Error unregistering node '{}': Exception occured while "
+                "waiting to be able to proceed: {}".format(machine['uuid'], e)
             )
 
         for nic in _normalize_port_list(nics):
@@ -353,21 +343,16 @@ class BaremetalCloudMixin:
 
             Example patch construction::
 
-                patch=[]
-                patch.append({
-                    'op': 'remove',
-                    'path': '/instance_info'
-                })
-                patch.append({
-                    'op': 'replace',
-                    'path': '/name',
-                    'value': 'newname'
-                })
-                patch.append({
-                    'op': 'add',
-                    'path': '/driver_info/username',
-                    'value': 'administrator'
-                })
+                patch = []
+                patch.append({'op': 'remove', 'path': '/instance_info'})
+                patch.append({'op': 'replace', 'path': '/name', 'value': 'newname'})
+                patch.append(
+                    {
+                        'op': 'add',
+                        'path': '/driver_info/username',
+                        'value': 'administrator',
+                    }
+                )
 
         :returns: Current state of the node.
         :rtype: :class:`~openstack.baremetal.v1.node.Node`.
@@ -395,7 +380,7 @@ class BaremetalCloudMixin:
         machine = self.get_machine(name_or_id)
         if not machine:
             raise exceptions.SDKException(
-                "Machine update failed to find Machine: %s. " % name_or_id
+                f"Machine update failed to find Machine: {name_or_id}. "
             )
 
         new_config = dict(machine._to_munch(), **attrs)
@@ -407,8 +392,7 @@ class BaremetalCloudMixin:
         except Exception as e:
             raise exceptions.SDKException(
                 "Machine update failed - Error generating JSON patch object "
-                "for submission to the API. Machine: %s Error: %s"
-                % (name_or_id, e)
+                f"for submission to the API. Machine: {name_or_id} Error: {e}"
             )
 
         if not patch:
@@ -427,7 +411,7 @@ class BaremetalCloudMixin:
         :return: Nothing.
         """
         machine = self.get_machine(name_or_id)
-        port = self.get_port(port_name_or_id)
+        port = self.network.find_port(port_name_or_id, ignore_missing=False)
         self.baremetal.attach_vif_to_node(machine, port['id'])
 
     def detach_port_from_machine(self, name_or_id, port_name_or_id):
@@ -439,7 +423,7 @@ class BaremetalCloudMixin:
         :return: Nothing.
         """
         machine = self.get_machine(name_or_id)
-        port = self.get_port(port_name_or_id)
+        port = self.network.find_port(port_name_or_id, ignore_missing=False)
         self.baremetal.detach_vif_from_node(machine, port['id'])
 
     def list_ports_attached_to_machine(self, name_or_id):
@@ -451,7 +435,10 @@ class BaremetalCloudMixin:
         """
         machine = self.get_machine(name_or_id)
         vif_ids = self.baremetal.list_node_vifs(machine)
-        return [self.get_port(vif) for vif in vif_ids]
+        return [
+            self.network.find_port(vif, ignore_missing=False)
+            for vif in vif_ids
+        ]
 
     def validate_machine(self, name_or_id, for_deploy=True):
         """Validate parameters of the machine.
@@ -463,15 +450,15 @@ class BaremetalCloudMixin:
         :raises: :exc:`~openstack.exceptions.ValidationException`
         """
         if for_deploy:
-            ifaces = ('boot', 'deploy', 'management', 'power')
+            ifaces = ['boot', 'deploy', 'management', 'power']
         else:
-            ifaces = ('power',)
+            ifaces = ['power']
         self.baremetal.validate_node(name_or_id, required=ifaces)
 
     def validate_node(self, uuid):
         warnings.warn(
             'validate_node is deprecated, please use validate_machine instead',
-            os_warnings.OpenStackDeprecationWarning,
+            os_warnings.RemovedInSDK50Warning,
         )
         self.baremetal.validate_node(uuid)
 
@@ -612,7 +599,7 @@ class BaremetalCloudMixin:
         warnings.warn(
             "The set_node_instance_info call is deprecated, "
             "use patch_machine or update_machine instead",
-            os_warnings.OpenStackDeprecationWarning,
+            os_warnings.RemovedInSDK50Warning,
         )
         return self.patch_machine(uuid, patch)
 
@@ -620,7 +607,7 @@ class BaremetalCloudMixin:
         warnings.warn(
             "The purge_node_instance_info call is deprecated, "
             "use patch_machine or update_machine instead",
-            os_warnings.OpenStackDeprecationWarning,
+            os_warnings.RemovedInSDK50Warning,
         )
         return self.patch_machine(
             uuid, dict(path='/instance_info', op='remove')
@@ -639,6 +626,6 @@ class BaremetalCloudMixin:
             "The wait_for_baremetal_node_lock call is deprecated "
             "in favor of wait_for_node_reservation on the baremetal "
             "proxy",
-            os_warnings.OpenStackDeprecationWarning,
+            os_warnings.RemovedInSDK50Warning,
         )
         self.baremetal.wait_for_node_reservation(node, timeout)
