@@ -12,31 +12,28 @@
 
 import warnings
 
-from openstack.block_storage.v3._proxy import Proxy
-from openstack.block_storage.v3 import quota_set as _qs
 from openstack.cloud import _utils
+from openstack.cloud import openstackcloud
 from openstack import exceptions
+from openstack import resource
 from openstack import warnings as os_warnings
 
 
-class BlockStorageCloudMixin:
-    block_storage: Proxy
-
-    # TODO(stephenfin): Remove 'cache' in a future major version
-    def list_volumes(self, cache=True):
+class BlockStorageCloudMixin(openstackcloud._OpenStackCloudMixin):
+    def list_volumes(self, cache=None):
         """List all available volumes.
 
         :param cache: **DEPRECATED** This parameter no longer does anything.
         :returns: A list of volume ``Volume`` objects.
         """
-        warnings.warn(
-            "the 'cache' argument is deprecated and no longer does anything; "
-            "consider removing it from calls",
-            os_warnings.OpenStackDeprecationWarning,
-        )
+        if cache is not None:
+            warnings.warn(
+                "the 'cache' argument is deprecated and no longer does "
+                "anything; consider removing it from calls",
+                os_warnings.RemovedInSDK50Warning,
+            )
         return list(self.block_storage.volumes())
 
-    # TODO(stephenfin): Remove 'get_extra' in a future major version
     def list_volume_types(self, get_extra=None):
         """List all available volume types.
 
@@ -48,7 +45,7 @@ class BlockStorageCloudMixin:
             warnings.warn(
                 "the 'get_extra' argument is deprecated and no longer does "
                 "anything; consider removing it from calls",
-                os_warnings.OpenStackDeprecationWarning,
+                os_warnings.RemovedInSDK50Warning,
             )
         return list(self.block_storage.types())
 
@@ -61,12 +58,7 @@ class BlockStorageCloudMixin:
             further filtering. Elements of this dictionary may, themselves, be
             dictionaries. Example::
 
-                {
-                  'last_name': 'Smith',
-                  'other': {
-                      'gender': 'Female'
-                  }
-                }
+                {'last_name': 'Smith', 'other': {'gender': 'Female'}}
 
             OR
 
@@ -96,12 +88,7 @@ class BlockStorageCloudMixin:
             further filtering. Elements of this dictionary may, themselves, be
             dictionaries. Example::
 
-                {
-                  'last_name': 'Smith',
-                  'other': {
-                      'gender': 'Female'
-                  }
-                }
+                {'last_name': 'Smith', 'other': {'gender': 'Female'}}
 
             OR
 
@@ -144,15 +131,18 @@ class BlockStorageCloudMixin:
             wait = True
 
         if image:
-            image_obj = self.get_image(image)
-            if not image_obj:
-                raise exceptions.SDKException(
-                    "Image {image} was requested as the basis for a new"
-                    " volume, but was not found on the cloud".format(
-                        image=image
+            if isinstance(image, dict):
+                if not isinstance(image, resource.Resource):
+                    warnings.warn(
+                        "Support for passing image as a raw dict has "
+                        "been deprecated for removal. Consider passing a "
+                        "string name or ID or an Image object instead.",
+                        os_warnings.RemovedInSDK60Warning,
                     )
-                )
-            kwargs['imageRef'] = image_obj['id']
+                kwargs['imageRef'] = image['id']
+            else:  # object
+                image_obj = self.image.find_image(image, ignore_missing=False)
+                kwargs['imageRef'] = image_obj['id']
         kwargs = self._get_volume_kwargs(kwargs)
         kwargs['size'] = size
 
@@ -179,7 +169,7 @@ class BlockStorageCloudMixin:
 
         volume = self.get_volume(name_or_id)
         if not volume:
-            raise exceptions.SDKException("Volume %s not found." % name_or_id)
+            raise exceptions.SDKException(f"Volume {name_or_id} not found.")
 
         volume = self.block_storage.update_volume(volume, **kwargs)
 
@@ -203,9 +193,7 @@ class BlockStorageCloudMixin:
 
         if not volume:
             raise exceptions.SDKException(
-                "Volume {name_or_id} does not exist".format(
-                    name_or_id=name_or_id
-                )
+                f"Volume {name_or_id} does not exist"
             )
 
         self.block_storage.set_volume_bootable_status(volume, bootable)
@@ -231,8 +219,9 @@ class BlockStorageCloudMixin:
         :raises: :class:`~openstack.exceptions.SDKException` on operation
             error.
         """
-        volume = self.block_storage.find_volume(name_or_id)
-
+        volume = self.block_storage.find_volume(
+            name_or_id, ignore_missing=True
+        )
         if not volume:
             self.log.debug(
                 "Volume %(name_or_id)s does not exist",
@@ -251,14 +240,22 @@ class BlockStorageCloudMixin:
 
         return True
 
-    # TODO(stephenfin): Remove 'cache' in a future major version
-    def get_volumes(self, server, cache=True):
+    def get_volumes(self, server, cache=None):
         """Get volumes for a server.
 
         :param server: The server to fetch volumes for.
         :param cache: **DEPRECATED** This parameter no longer does anything.
         :returns: A list of volume ``Volume`` objects.
         """
+        if cache is not None:
+            warnings.warn(
+                "the 'cache' argument is deprecated and no longer does "
+                "anything; consider removing it from calls",
+                os_warnings.RemovedInSDK50Warning,
+            )
+            # avoid spamming warnings
+            cache = None
+
         volumes = []
         for volume in self.list_volumes(cache=cache):
             for attach in volume['attachments']:
@@ -271,13 +268,13 @@ class BlockStorageCloudMixin:
 
         :param name_or_id: (optional) Project name or ID to get limits for
             if different from the current project
-        :returns: The volume ``Limit`` object if found, else None.
+        :returns: The volume ``Limits`` object if found, else None.
         """
         params = {}
         if name_or_id:
-            project = self.get_project(name_or_id)
-            if not project:
-                raise exceptions.SDKException("project does not exist")
+            project = self.identity.find_project(
+                name_or_id, ignore_missing=False
+            )
             params['project'] = project
         return self.block_storage.get_limits(**params)
 
@@ -372,14 +369,16 @@ class BlockStorageCloudMixin:
         dev = self.get_volume_attach_device(volume, server['id'])
         if dev:
             raise exceptions.SDKException(
-                "Volume %s already attached to server %s on device %s"
-                % (volume['id'], server['id'], dev)
+                "Volume {} already attached to server {} on device {}".format(
+                    volume['id'], server['id'], dev
+                )
             )
 
         if volume['status'] != 'available':
             raise exceptions.SDKException(
-                "Volume %s is not available. Status is '%s'"
-                % (volume['id'], volume['status'])
+                "Volume {} is not available. Status is '{}'".format(
+                    volume['id'], volume['status']
+                )
             )
 
         payload = {}
@@ -471,12 +470,7 @@ class BlockStorageCloudMixin:
             further filtering. Elements of this dictionary may, themselves, be
             dictionaries. Example::
 
-                {
-                  'last_name': 'Smith',
-                  'other': {
-                      'gender': 'Female'
-                  }
-                }
+                {'last_name': 'Smith', 'other': {'gender': 'Female'}}
 
             OR
 
@@ -546,12 +540,7 @@ class BlockStorageCloudMixin:
             further filtering. Elements of this dictionary may, themselves, be
             dictionaries. Example::
 
-                {
-                  'last_name': 'Smith',
-                  'other': {
-                      'gender': 'Female'
-                  }
-                }
+                {'last_name': 'Smith', 'other': {'gender': 'Female'}}
 
             OR
 
@@ -574,7 +563,7 @@ class BlockStorageCloudMixin:
                 {
                     'name': 'my-volume-snapshot',
                     'volume_id': 'e126044c-7b4c-43be-a32a-c9cbbc9ddb56',
-                    'all_tenants': 1
+                    'all_tenants': 1,
                 }
 
         :returns: A list of volume ``Snapshot`` objects.
@@ -594,7 +583,7 @@ class BlockStorageCloudMixin:
                     'name': 'my-volume-backup',
                     'status': 'available',
                     'volume_id': 'e126044c-7b4c-43be-a32a-c9cbbc9ddb56',
-                    'all_tenants': 1
+                    'all_tenants': 1,
                 }
 
         :returns: A list of volume ``Backup`` objects.
@@ -675,12 +664,7 @@ class BlockStorageCloudMixin:
             further filtering. Elements of this dictionary may, themselves, be
             dictionaries. Example::
 
-                {
-                  'last_name': 'Smith',
-                  'other': {
-                      'gender': 'Female'
-                  }
-                }
+                {'last_name': 'Smith', 'other': {'gender': 'Female'}}
 
             OR
 
@@ -702,12 +686,7 @@ class BlockStorageCloudMixin:
             further filtering. Elements of this dictionary may, themselves, be
             dictionaries. Example::
 
-                {
-                  'last_name': 'Smith',
-                  'other': {
-                      'gender': 'Female'
-                  }
-                }
+                {'last_name': 'Smith', 'other': {'gender': 'Female'}}
 
             OR
 
@@ -729,12 +708,7 @@ class BlockStorageCloudMixin:
             further filtering. Elements of this dictionary may, themselves, be
             dictionaries. Example::
 
-                {
-                  'last_name': 'Smith',
-                  'other': {
-                      'gender': 'Female'
-                  }
-                }
+                {'last_name': 'Smith', 'other': {'gender': 'Female'}}
 
             OR
 
@@ -748,7 +722,6 @@ class BlockStorageCloudMixin:
         volume_backups = self.list_volume_backups()
         return _utils._filter_list(volume_backups, name_or_id, filters)
 
-    # TODO(stephenfin): Remove 'get_extra' in a future major version
     def search_volume_types(
         self,
         name_or_id=None,
@@ -762,12 +735,7 @@ class BlockStorageCloudMixin:
             further filtering. Elements of this dictionary may, themselves, be
             dictionaries. Example::
 
-                {
-                  'last_name': 'Smith',
-                  'other': {
-                      'gender': 'Female'
-                  }
-                }
+                {'last_name': 'Smith', 'other': {'gender': 'Female'}}
 
             OR
 
@@ -778,7 +746,13 @@ class BlockStorageCloudMixin:
 
         :returns: A list of volume ``Type`` objects, if any are found.
         """
-        volume_types = self.list_volume_types(get_extra=get_extra)
+        if get_extra is not None:
+            warnings.warn(
+                "the 'get_extra' argument is deprecated and no longer does "
+                "anything; consider removing it from calls",
+                os_warnings.RemovedInSDK50Warning,
+            )
+        volume_types = self.list_volume_types()
         return _utils._filter_list(volume_types, name_or_id, filters)
 
     def get_volume_type_access(self, name_or_id):
@@ -792,7 +766,7 @@ class BlockStorageCloudMixin:
         volume_type = self.get_volume_type(name_or_id)
         if not volume_type:
             raise exceptions.SDKException(
-                "VolumeType not found: %s" % name_or_id
+                f"VolumeType not found: {name_or_id}"
             )
 
         return self.block_storage.get_type_access(volume_type)
@@ -812,7 +786,7 @@ class BlockStorageCloudMixin:
         volume_type = self.get_volume_type(name_or_id)
         if not volume_type:
             raise exceptions.SDKException(
-                "VolumeType not found: %s" % name_or_id
+                f"VolumeType not found: {name_or_id}"
             )
 
         self.block_storage.add_type_access(volume_type, project_id)
@@ -830,7 +804,7 @@ class BlockStorageCloudMixin:
         volume_type = self.get_volume_type(name_or_id)
         if not volume_type:
             raise exceptions.SDKException(
-                "VolumeType not found: %s" % name_or_id
+                f"VolumeType not found: {name_or_id}"
             )
         self.block_storage.remove_type_access(volume_type, project_id)
 
@@ -844,12 +818,9 @@ class BlockStorageCloudMixin:
         :raises: :class:`~openstack.exceptions.SDKException` if the resource to
             set the quota does not exist.
         """
+        project = self.identity.find_project(name_or_id, ignore_missing=False)
 
-        proj = self.identity.find_project(name_or_id, ignore_missing=False)
-
-        self.block_storage.update_quota_set(
-            _qs.QuotaSet(project_id=proj.id), **kwargs
-        )
+        self.block_storage.update_quota_set(project=project, **kwargs)
 
     def get_volume_quotas(self, name_or_id):
         """Get volume quotas for a project
