@@ -12,16 +12,14 @@
 
 import concurrent.futures
 import urllib.parse
+import warnings
 
 import keystoneauth1.exceptions
 
 from openstack.cloud import _utils
+from openstack.cloud import openstackcloud
 from openstack import exceptions
-from openstack.object_store.v1._proxy import Proxy
-
-DEFAULT_OBJECT_SEGMENT_SIZE = 1073741824  # 1GB
-# This halves the current default for Swift
-DEFAULT_MAX_FILE_SIZE = (5 * 1024 * 1024 * 1024 + 2) / 2
+from openstack import warnings as os_warnings
 
 
 OBJECT_CONTAINER_ACLS = {
@@ -30,11 +28,8 @@ OBJECT_CONTAINER_ACLS = {
 }
 
 
-class ObjectStoreCloudMixin:
-    object_store: Proxy
-
-    # TODO(stephenfin): Remove 'full_listing' as it's a noop
-    def list_containers(self, full_listing=True, prefix=None):
+class ObjectStoreCloudMixin(openstackcloud._OpenStackCloudMixin):
+    def list_containers(self, full_listing=None, prefix=None):
         """List containers.
 
         :param full_listing: Ignored. Present for backwards compat
@@ -44,6 +39,12 @@ class ObjectStoreCloudMixin:
         :raises: :class:`~openstack.exceptions.SDKException` on operation
             error.
         """
+        if full_listing is not None:
+            warnings.warn(
+                "The 'full_listing' field is unnecessary and will be removed "
+                "in a future release.",
+                os_warnings.RemovedInSDK60Warning,
+            )
         return list(self.object_store.containers(prefix=prefix))
 
     def search_containers(self, name=None, filters=None):
@@ -113,11 +114,9 @@ class ObjectStoreCloudMixin:
             return False
         except exceptions.ConflictException:
             raise exceptions.SDKException(
-                'Attempt to delete container {container} failed. The'
-                ' container is not empty. Please delete the objects'
-                ' inside it before deleting the container'.format(
-                    container=name
-                )
+                f'Attempt to delete container {name} failed. The '
+                f'container is not empty. Please delete the objects '
+                f'inside it before deleting the container'
             )
 
     def update_container(self, name, headers):
@@ -141,8 +140,8 @@ class ObjectStoreCloudMixin:
         """
         if access not in OBJECT_CONTAINER_ACLS:
             raise exceptions.SDKException(
-                "Invalid container access specified: %s.  Must be one of %s"
-                % (access, list(OBJECT_CONTAINER_ACLS.keys()))
+                f"Invalid container access specified: {access}. "
+                f"Must be one of {list(OBJECT_CONTAINER_ACLS.keys())}"
             )
         return self.object_store.set_container_metadata(
             name, read_ACL=OBJECT_CONTAINER_ACLS[access], refresh=refresh
@@ -158,7 +157,7 @@ class ObjectStoreCloudMixin:
         """
         container = self.get_container(name, skip_cache=True)
         if not container:
-            raise exceptions.SDKException("Container not found: %s" % name)
+            raise exceptions.SDKException(f"Container not found: {name}")
         acl = container.read_ACL or ''
         for key, value in OBJECT_CONTAINER_ACLS.items():
             # Convert to string for the comparison because swiftclient
@@ -167,7 +166,7 @@ class ObjectStoreCloudMixin:
             if str(acl) == str(value):
                 return key
         raise exceptions.SDKException(
-            "Could not determine container access for ACL: %s." % acl
+            f"Could not determine container access for ACL: {acl}."
         )
 
     def get_object_capabilities(self):
@@ -422,13 +421,9 @@ class ObjectStoreCloudMixin:
     def _get_object_endpoint(self, container, obj=None, query_string=None):
         endpoint = urllib.parse.quote(container)
         if obj:
-            endpoint = '{endpoint}/{object}'.format(
-                endpoint=endpoint, object=urllib.parse.quote(obj)
-            )
+            endpoint = f'{endpoint}/{urllib.parse.quote(obj)}'
         if query_string:
-            endpoint = '{endpoint}?{query_string}'.format(
-                endpoint=endpoint, query_string=query_string
-            )
+            endpoint = f'{endpoint}?{query_string}'
         return endpoint
 
     def stream_object(
@@ -453,11 +448,10 @@ class ObjectStoreCloudMixin:
             error.
         """
         try:
-            for ret in self.object_store.stream_object(
+            yield from self.object_store.stream_object(
                 obj, container, chunk_size=resp_chunk_size
-            ):
-                yield ret
-        except exceptions.ResourceNotFound:
+            )
+        except exceptions.NotFoundException:
             return
 
     def get_object(
@@ -499,7 +493,7 @@ class ObjectStoreCloudMixin:
             headers = {k.lower(): v for k, v in obj._last_headers.items()}
             return (headers, obj.data)
 
-        except exceptions.ResourceNotFound:
+        except exceptions.NotFoundException:
             return None
 
     def _wait_for_futures(self, futures, raise_on_error=True):
@@ -517,9 +511,7 @@ class ObjectStoreCloudMixin:
                 keystoneauth1.exceptions.RetriableConnectionFailure,
                 exceptions.HttpException,
             ) as e:
-                error_text = "Exception processing async task: {}".format(
-                    str(e)
-                )
+                error_text = f"Exception processing async task: {str(e)}"
                 if raise_on_error:
                     self.log.exception(error_text)
                     raise
