@@ -69,6 +69,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         "key_name",
         "launch_index",
         "launched_at",
+        "locked",
         "locked_by",
         "name",
         "node",
@@ -280,13 +281,18 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         self,
         requires_id=True,
         prepend_key=True,
+        patch=False,
         base_path=None,
+        params=None,
         **kwargs,
     ):
         request = super()._prepare_request(
             requires_id=requires_id,
             prepend_key=prepend_key,
+            patch=patch,
             base_path=base_path,
+            params=params,
+            **kwargs,
         )
 
         server_body = request.body[self.resource_key]
@@ -836,13 +842,22 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         body = {'unshelve': data or None}
         self._action(session, body)
 
-    def migrate(self, session):
+    def migrate(self, session, *, host=None):
         """Migrate the server.
 
         :param session: The session to use for making this request.
+        :param host: The host to migrate the server to. (Optional)
         :returns: None
         """
-        body = {"migrate": None}
+        if host and not utils.supports_microversion(session, '2.56'):
+            raise ValueError(
+                "The 'host' option is only supported on microversion 2.56 or "
+                "greater."
+            )
+
+        body: ty.Dict[str, ty.Any] = {"migrate": None}
+        if host:
+            body["migrate"] = {"host": host}
         self._action(session, body)
 
     def trigger_crash_dump(self, session):
@@ -879,7 +894,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         """
         action = CONSOLE_TYPE_ACTION_MAPPING.get(console_type)
         if not action:
-            raise ValueError("Unsupported console type %s" % console_type)
+            raise ValueError(f"Unsupported console type {console_type}")
         body = {action: {'type': console_type}}
         resp = self._action(session, body)
         return resp.json().get('console')
@@ -952,12 +967,12 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
             body['host'] = host
             if not force:
                 raise ValueError(
-                    "Live migration on this cloud implies 'force'"
-                    " if the 'host' option has been given and it is not"
-                    " possible to disable. It is recommended to not use 'host'"
-                    " at all on this cloud as it is inherently unsafe, but if"
-                    " it is unavoidable, please supply 'force=True' so that it"
-                    " is clear you understand the risks."
+                    "Live migration on this cloud implies 'force' "
+                    "if the 'host' option has been given and it is not "
+                    "possible to disable. It is recommended to not use 'host' "
+                    "at all on this cloud as it is inherently unsafe, but if "
+                    "it is unavoidable, please supply 'force=True' so that it "
+                    "is clear you understand the risks."
                 )
         self._action(
             session,
@@ -979,8 +994,8 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         }
         if block_migration == 'auto':
             raise ValueError(
-                "Live migration on this cloud does not support 'auto' as"
-                " a parameter to block_migration, but only True and False."
+                "Live migration on this cloud does not support 'auto' as "
+                "a parameter to block_migration, but only True and False."
             )
         body['block_migration'] = block_migration or False
         body['disk_over_commit'] = disk_over_commit or False
@@ -988,12 +1003,12 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
             body['host'] = host
             if not force:
                 raise ValueError(
-                    "Live migration on this cloud implies 'force'"
-                    " if the 'host' option has been given and it is not"
-                    " possible to disable. It is recommended to not use 'host'"
-                    " at all on this cloud as it is inherently unsafe, but if"
-                    " it is unavoidable, please supply 'force=True' so that it"
-                    " is clear you understand the risks."
+                    "Live migration on this cloud implies 'force' "
+                    "if the 'host' option has been given and it is not "
+                    "possible to disable. It is recommended to not use 'host' "
+                    "at all on this cloud as it is inherently unsafe, but if "
+                    "it is unavoidable, please supply 'force=True' so that it "
+                    "is clear you understand the risks."
                 )
         self._action(
             session,
