@@ -24,9 +24,9 @@ import sys
 import typing as ty
 import warnings
 
-import appdirs
 from keystoneauth1 import adapter
 from keystoneauth1 import loading
+import platformdirs
 import yaml
 
 from openstack import _log
@@ -37,9 +37,11 @@ from openstack.config import vendors
 from openstack import exceptions
 from openstack import warnings as os_warnings
 
-APPDIRS = appdirs.AppDirs('openstack', 'OpenStack', multipath='/etc')
-CONFIG_HOME = APPDIRS.user_config_dir
-CACHE_PATH = APPDIRS.user_cache_dir
+PLATFORMDIRS = platformdirs.PlatformDirs(
+    'openstack', 'OpenStack', multipath='/etc'
+)
+CONFIG_HOME = PLATFORMDIRS.user_config_dir
+CACHE_PATH = PLATFORMDIRS.user_cache_dir
 
 # snaps do set $HOME to something like
 # /home/$USER/snap/openstackclients/$SNAP_VERSION
@@ -56,7 +58,7 @@ else:
     )
 UNIX_SITE_CONFIG_HOME = '/etc/openstack'
 
-SITE_CONFIG_HOME = APPDIRS.site_config_dir
+SITE_CONFIG_HOME = PLATFORMDIRS.site_config_dir
 
 CONFIG_SEARCH_PATH = [
     os.getcwd(),
@@ -136,10 +138,10 @@ def _fix_argv(argv):
             overlap.extend(old)
     if overlap:
         raise exceptions.ConfigException(
-            "The following options were given: '{options}' which contain"
-            " duplicates except that one has _ and one has -. There is"
-            " no sane way for us to know what you're doing. Remove the"
-            " duplicate option and try again".format(options=','.join(overlap))
+            "The following options were given: '{options}' which contain "
+            "duplicates except that one has _ and one has -. There is "
+            "no sane way for us to know what you're doing. Remove the "
+            "duplicate option and try again".format(options=','.join(overlap))
         )
 
 
@@ -208,15 +210,19 @@ class OpenStackConfig:
 
         # First, use a config file if it exists where expected
         self.config_filename, self.cloud_config = self._load_config_file()
-        _, secure_config = self._load_secure_file()
+        if self.config_filename:
+            self._validate_config_file(self.config_filename, self.cloud_config)
+
+        secure_config_filename, secure_config = self._load_secure_file()
         if secure_config:
+            self._validate_config_file(secure_config_filename, secure_config)
             self.cloud_config = _util.merge_clouds(
                 self.cloud_config, secure_config
             )
 
         if not self.cloud_config:
             self.cloud_config = {'clouds': {}}
-        if 'clouds' not in self.cloud_config:
+        elif 'clouds' not in self.cloud_config:
             self.cloud_config['clouds'] = {}
 
         # Save the other config
@@ -258,12 +264,11 @@ class OpenStackConfig:
         self.envvar_key = self._get_envvar('OS_CLOUD_NAME', 'envvars')
         if self.envvar_key in self.cloud_config['clouds']:
             raise exceptions.ConfigException(
-                '"{0}" defines a cloud named "{1}", but'
-                ' OS_CLOUD_NAME is also set to "{1}". Please rename'
-                ' either your environment based cloud, or one of your'
-                ' file-based clouds.'.format(
-                    self.config_filename, self.envvar_key
-                )
+                f'{self.config_filename!r} defines a cloud named '
+                f'{self.envvar_key!r}, but OS_CLOUD_NAME is also set to '
+                f'{self.envvar_key!r}. '
+                f'Please rename either your environment-based cloud, '
+                f'or one of your file-based clouds.'
             )
 
         self.default_cloud = self._get_envvar('OS_CLOUD')
@@ -409,15 +414,13 @@ class OpenStackConfig:
             ret[newkey] = os.environ[k]
         # If the only environ keys are selectors or behavior modification,
         # don't return anything
-        selectors = set(
-            [
-                'OS_CLOUD',
-                'OS_REGION_NAME',
-                'OS_CLIENT_CONFIG_FILE',
-                'OS_CLIENT_SECURE_FILE',
-                'OS_CLOUD_NAME',
-            ]
-        )
+        selectors = {
+            'OS_CLOUD',
+            'OS_REGION_NAME',
+            'OS_CLIENT_CONFIG_FILE',
+            'OS_CLIENT_SECURE_FILE',
+            'OS_CLOUD_NAME',
+        }
         if set(environkeys) - selectors:
             return ret
         return None
@@ -454,18 +457,37 @@ class OpenStackConfig:
         for path in filelist:
             if os.path.exists(path):
                 try:
-                    with open(path, 'r') as f:
+                    with open(path) as f:
                         if path.endswith('json'):
                             return path, json.load(f)
                         else:
                             return path, yaml.safe_load(f)
-                except IOError as e:
+                except OSError as e:
                     if e.errno == errno.EACCES:
                         # Can't access file so let's continue to the next
                         # file
                         continue
         return (None, {})
 
+    def _validate_config_file(self, path: str, data: ty.Any) -> bool:
+        """Validate config file contains a clouds entry.
+
+        All config files should have a 'clouds' key at a minimum.
+        """
+        if not isinstance(data, dict):
+            raise exceptions.ConfigException(
+                f'Configuration file {path} is empty or not a valid mapping'
+            )
+
+        if 'clouds' not in data:
+            # TODO(stephenfin): This should probably be an error at some point
+            self.log.warning(
+                "Configuration file %s does not contain a 'clouds' key", path
+            )
+            return False
+
+        return True
+
     def _expand_region_name(self, region_name):
         return {'name': region_name, 'values': {}}
 
@@ -478,7 +500,7 @@ class OpenStackConfig:
                     region
                 ):
                     raise exceptions.ConfigException(
-                        'Invalid region entry at: %s' % region
+                        f'Invalid region entry at: {region}'
                     )
                 if 'values' not in region:
                     region['values'] = {}
@@ -541,9 +563,9 @@ class OpenStackConfig:
                 return region
 
         raise exceptions.ConfigException(
-            'Region {region_name} is not a valid region name for cloud'
-            ' {cloud}. Valid choices are {region_list}. Please note that'
-            ' region names are case sensitive.'.format(
+            'Region {region_name} is not a valid region name for cloud '
+            '{cloud}. Valid choices are {region_list}. Please note that '
+            'region names are case sensitive.'.format(
                 region_name=region_name,
                 region_list=','.join([r['name'] for r in regions]),
                 cloud=cloud,
@@ -558,9 +580,7 @@ class OpenStackConfig:
 
         # Only validate cloud name if one was given
         if name and name not in self.cloud_config['clouds']:
-            raise exceptions.ConfigException(
-                "Cloud {name} was not found.".format(name=name)
-            )
+            raise exceptions.ConfigException(f"Cloud {name} was not found.")
 
         our_cloud = self.cloud_config['clouds'].get(name, dict())
         if profile:
@@ -617,16 +637,14 @@ class OpenStackConfig:
                     )
                 elif status == 'shutdown':
                     raise exceptions.ConfigException(
-                        "{profile_name} references a cloud that no longer"
-                        " exists: {message}".format(
-                            profile_name=profile_name, message=message
-                        )
+                        f"{profile_name} references a cloud that no longer "
+                        f"exists: {message}"
                     )
                 _auth_update(cloud, profile_data)
             else:
                 # Can't find the requested vendor config, go about business
                 warnings.warn(
-                    f"Couldn't find the vendor profile {profile_name} for"
+                    f"Couldn't find the vendor profile {profile_name} for "
                     f"the cloud {name}",
                     os_warnings.ConfigurationWarning,
                 )
@@ -644,8 +662,8 @@ class OpenStackConfig:
         for net in networks:
             if value and net[key]:
                 raise exceptions.ConfigException(
-                    "Duplicate network entries for {key}: {net1} and {net2}."
-                    " Only one network can be flagged with {key}".format(
+                    "Duplicate network entries for {key}: {net1} and {net2}. "
+                    "Only one network can be flagged with {key}".format(
                         key=key, net1=value['name'], net2=net['name']
                     )
                 )
@@ -684,9 +702,9 @@ class OpenStackConfig:
             external = key.startswith('external')
             if key in cloud and 'networks' in cloud:
                 raise exceptions.ConfigException(
-                    "Both {key} and networks were specified in the config."
-                    " Please remove {key} from the config and use the network"
-                    " list to configure network behavior.".format(key=key)
+                    f"Both {key} and networks were specified in the config. "
+                    f"Please remove {key} from the config and use the network "
+                    f"list to configure network behavior."
                 )
             if key in cloud:
                 warnings.warn(
@@ -777,12 +795,15 @@ class OpenStackConfig:
         for target_key, possible_values in mappings.items():
             target = None
             for key in possible_values:
-                if key in cloud:
-                    target = str(cloud[key])
-                    del cloud[key]
+                # Prefer values from the 'auth' section
+                # as they may contain cli or environment overrides.
+                # See story 2010784 for context.
                 if key in cloud['auth']:
                     target = str(cloud['auth'][key])
                     del cloud['auth'][key]
+                if key in cloud:
+                    target = str(cloud[key])
+                    del cloud[key]
             if target:
                 cloud['auth'][target_key] = target
         return cloud
@@ -882,8 +903,8 @@ class OpenStackConfig:
             options, _args = parser.parse_known_args(argv)
             plugin_names = loading.get_available_plugin_names()
             raise exceptions.ConfigException(
-                "An invalid auth-type was specified: {auth_type}."
-                " Valid choices are: {plugin_names}.".format(
+                "An invalid auth-type was specified: {auth_type}. "
+                "Valid choices are: {plugin_names}.".format(
                     auth_type=options.os_auth_type,
                     plugin_names=",".join(plugin_names),
                 )
@@ -960,8 +981,13 @@ class OpenStackConfig:
                     )
         return clouds
 
-    # TODO(mordred) Backwards compat for OSC transition
-    get_all_clouds = get_all
+    def get_all_clouds(self):
+        warnings.warn(
+            "The 'get_all_clouds' method is a deprecated alias for "
+            "'get_clouds' and will be removed in a future release.",
+            os_warnings.RemovedInSDK60Warning,
+        )
+        return self.get_all()
 
     def _fix_args(self, args=None, argparse=None):
         """Massage the passed-in options
@@ -986,7 +1012,7 @@ class OpenStackConfig:
         os_args = dict()
         new_args = dict()
         for key, val in iter(args.items()):
-            if type(args[key]) == dict:
+            if type(args[key]) is dict:
                 # dive into the auth dict
                 new_args[key] = self._fix_args(args[key])
                 continue
@@ -1319,8 +1345,20 @@ class OpenStackConfig:
             influxdb_config=influxdb_config,
         )
 
-    # TODO(mordred) Backwards compat for OSC transition
-    get_one_cloud = get_one
+    def get_one_cloud(
+        self, cloud=None, validate=True, argparse=None, **kwargs
+    ):
+        warnings.warn(
+            "The 'get_one_cloud' method is a deprecated alias for 'get_one' "
+            "and will be removed in a future release.",
+            os_warnings.RemovedInSDK60Warning,
+        )
+        return self.get_one(
+            cloud=cloud,
+            validate=validate,
+            argparse=argparse,
+            **kwargs,
+        )
 
     def get_one_cloud_osc(
         self, cloud=None, validate=True, argparse=None, **kwargs
@@ -1435,7 +1473,7 @@ class OpenStackConfig:
         try:
             with open(config_file) as fh:
                 cur_config = yaml.safe_load(fh)
-        except IOError as e:
+        except OSError as e:
             # Not no such file
             if e.errno != 2:
                 raise
