@@ -12,6 +12,7 @@
 
 from openstack.dns.v2 import floating_ip as _fip
 from openstack.dns.v2 import recordset as _rs
+from openstack.dns.v2 import service_status as _svc_status
 from openstack.dns.v2 import zone as _zone
 from openstack.dns.v2 import zone_export as _zone_export
 from openstack.dns.v2 import zone_import as _zone_import
@@ -24,6 +25,7 @@ class Proxy(proxy.Proxy):
     _resource_registry = {
         "floating_ip": _fip.FloatingIP,
         "recordset": _rs.Recordset,
+        "service_status": _svc_status.ServiceStatus,
         "zone": _zone.Zone,
         "zone_export": _zone_export.ZoneExport,
         "zone_import": _zone_import.ZoneImport,
@@ -77,7 +79,7 @@ class Proxy(proxy.Proxy):
         :param zone: The value can be the ID of a zone
             or a :class:`~openstack.dns.v2.zone.Zone` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the zone does not exist.
             When set to ``True``, no exception will be set when attempting to
             delete a nonexistent zone.
@@ -111,7 +113,7 @@ class Proxy(proxy.Proxy):
 
         :param name_or_id: The name or ID of a zone
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the zone does not exist.
             When set to ``True``, no exception will be set when attempting
             to delete a nonexistent zone.
@@ -221,7 +223,7 @@ class Proxy(proxy.Proxy):
         :param zone: The value can be the ID of a zone
             or a :class:`~openstack.dns.v2.zone.Zone` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the zone does not exist. When set to ``True``, no exception will
             be set when attempting to delete a nonexistent zone.
 
@@ -242,7 +244,7 @@ class Proxy(proxy.Proxy):
             or a :class:`~openstack.dns.v2.zone.Zone` instance.
         :param name_or_id: The name or ID of a zone
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the zone does not exist.
             When set to ``True``, no exception will be set when attempting
             to delete a nonexistent zone.
@@ -303,7 +305,7 @@ class Proxy(proxy.Proxy):
         :param zone_import: The value can be the ID of a zone import
             or a :class:`~openstack.dns.v2.zone_import.ZoneImport` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the zone does not exist.
             When set to ``True``, no exception will be set when attempting to
             delete a nonexistent zone.
@@ -380,7 +382,7 @@ class Proxy(proxy.Proxy):
         :param zone_export: The value can be the ID of a zone import
             or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the zone does not exist.
             When set to ``True``, no exception will be set when attempting to
             delete a nonexistent zone.
@@ -510,7 +512,7 @@ class Proxy(proxy.Proxy):
             or a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
             instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the zone does not exist.
             When set to ``True``, no exception will be set when attempting to
             delete a nonexistent zone.
@@ -561,7 +563,7 @@ class Proxy(proxy.Proxy):
 
     # ======== Zone Shares ========
     def zone_shares(self, zone, **query):
-        """Retrieve a generator of zone sharess
+        """Retrieve a generator of zone shares
 
         :param zone: The zone ID or a
             :class:`~openstack.dns.v2.zone.Zone` instance
@@ -600,7 +602,7 @@ class Proxy(proxy.Proxy):
             or a :class:`~openstack.dns.v2.zone.Zone` instance.
         :param zone_share_id: The zone share ID
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the zone share does not exist.
             When set to ``True``,  None will be returned when attempting to
             find a nonexistent zone share.
@@ -641,7 +643,7 @@ class Proxy(proxy.Proxy):
             share or a :class:`~openstack.dns.v2.zone_share.ZoneShare` instance
             that the zone share belongs to.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the zone share does not exist.
             When set to ``True``, no exception will be set when attempting to
             delete a nonexistent zone share.
@@ -656,6 +658,26 @@ class Proxy(proxy.Proxy):
             zone_id=zone_obj.id,
         )
 
+    # ======== Service Statuses ========
+    def service_statuses(self):
+        """Retrieve a generator of service statuses
+
+        :returns: A generator of service statuses
+            :class:`~openstack.dns.v2.service_status.ServiceStatus` instances.
+        """
+        return self._list(_svc_status.ServiceStatus)
+
+    def get_service_status(self, service):
+        """Get a status of a service in the Designate system
+
+        :param service: The value can be the ID of a service
+            or a :class:`~openstack.dns.v2.service_status.ServiceStatus` instance.
+
+        :returns: ServiceStatus instance.
+        :rtype: :class:`~openstack.dns.v2.service_status.ServiceStatus`
+        """
+        return self._get(_svc_status.ServiceStatus, service)
+
     def _get_cleanup_dependencies(self):
         # DNS may depend on floating ip
         return {'dns': {'before': ['network']}}
