@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import hashlib
 import io
 
 from openstack import exceptions
@@ -22,7 +23,7 @@ def _verify_checksum(md5, checksum):
         digest = md5.hexdigest()
         if digest != checksum:
             raise exceptions.InvalidResponse(
-                "checksum mismatch: %s != %s" % (checksum, digest)
+                f"checksum mismatch: {checksum} != {digest}"
             )
 
 
@@ -41,8 +42,7 @@ class DownloadMixin:
         resource_response_key=None,
         microversion=None,
         **params,
-    ):
-        ...
+    ): ...
 
     def download(
         self,
@@ -69,7 +69,7 @@ class DownloadMixin:
             details = self.fetch(session)
             checksum = details.checksum
 
-        md5 = utils.md5(usedforsecurity=False)
+        md5 = hashlib.md5(usedforsecurity=False)
         if output:
             try:
                 if isinstance(output, io.IOBase):
@@ -85,9 +85,7 @@ class DownloadMixin:
 
                 return resp
             except Exception as e:
-                raise exceptions.SDKException(
-                    "Unable to download image: %s" % e
-                )
+                raise exceptions.SDKException(f"Unable to download image: {e}")
         # if we are returning the repsonse object, ensure that it
         # has the content-md5 header so that the caller doesn't
         # need to jump through the same hoops through which we
@@ -98,7 +96,7 @@ class DownloadMixin:
 
         if checksum is not None:
             _verify_checksum(
-                utils.md5(resp.content, usedforsecurity=False), checksum
+                hashlib.md5(resp.content, usedforsecurity=False), checksum
             )
         else:
             session.log.warning(
