@@ -40,13 +40,13 @@ _RAW_PROPERTIES = ('is_protected', 'protected', 'tags')
 
 def _get_name_and_filename(name, image_format):
     # See if name points to an existing file
-    if os.path.exists(name):
+    if os.path.exists(name) and os.path.isfile(name):
         # Neat. Easy enough
         return os.path.splitext(os.path.basename(name))[0], name
 
     # Try appending the disk format
     name_with_ext = '.'.join((name, image_format))
-    if os.path.exists(name_with_ext):
+    if os.path.exists(name_with_ext) and os.path.isfile(name):
         return os.path.basename(name), name_with_ext
 
     return name, None
@@ -90,7 +90,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.image.Image`
             instance.
         :param bool ignore_missing: When set to ``False``,
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the metadef namespace does not exist.
         :returns: ``None``
         """
@@ -101,7 +101,7 @@ class Proxy(proxy.Proxy):
         cache = self._get_resource(_cache.Cache, None)
         return cache.queue(self, image_id)
 
-    def clear_cache(self, target):
+    def clear_cache(self, target='both'):
         """Clear all images from cache, queue or both
 
         :param target: Specify which target you want to clear
@@ -333,8 +333,8 @@ class Proxy(proxy.Proxy):
                 validate_checksum=validate_checksum,
                 use_import=use_import,
                 stores=stores,
-                all_stores=stores,
-                all_stores_must_succeed=stores,
+                all_stores=all_stores,
+                all_stores_must_succeed=all_stores_must_succeed,
                 **image_kwargs,
             )
         else:
@@ -464,7 +464,7 @@ class Proxy(proxy.Proxy):
         if 'queued' != image.status:
             raise exceptions.SDKException(
                 'Image stage is only possible for images in the queued state. '
-                'Current state is {status}'.format(status=image.status)
+                f'Current state is {image.status}'
             )
 
         if filename:
@@ -506,7 +506,7 @@ class Proxy(proxy.Proxy):
         """
         warnings.warn(
             "upload_image is deprecated. Use create_image instead.",
-            os_warnings.OpenStackDeprecationWarning,
+            os_warnings.RemovedInSDK50Warning,
         )
         # container_format and disk_format are required to be set
         # on the image by the time upload_image is called, but they're not
@@ -598,9 +598,7 @@ class Proxy(proxy.Proxy):
             self.log.debug("Image creation failed", exc_info=True)
             raise
         except Exception as e:
-            raise exceptions.SDKException(
-                "Image creation failed: {message}".format(message=str(e))
-            )
+            raise exceptions.SDKException(f"Image creation failed: {str(e)}")
 
     def _make_v2_image_params(self, meta, properties):
         ret: ty.Dict = {}
@@ -696,9 +694,9 @@ class Proxy(proxy.Proxy):
     ):
         if not self._connection.has_service('object-store'):
             raise exceptions.SDKException(
-                "The cloud {cloud} is configured to use tasks for image "
+                f"The cloud {self._connection.config.name} is configured to use tasks for image "
                 "upload, but no object-store service is available. "
-                "Aborting.".format(cloud=self._connection.config.name)
+                "Aborting."
             )
 
         properties = image_kwargs.get('properties', {})
@@ -761,9 +759,7 @@ class Proxy(proxy.Proxy):
             except exceptions.ResourceFailure as e:
                 glance_task = self.get_task(glance_task)
                 raise exceptions.SDKException(
-                    "Image creation failed: {message}".format(
-                        message=e.message
-                    ),
+                    f"Image creation failed: {e.message}",
                     extra_data=glance_task,
                 )
             finally:
@@ -834,7 +830,7 @@ class Proxy(proxy.Proxy):
             image is associated with. If specified, the image will only be
             deleted from the specified store.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the image does not exist.
             When set to ``True``, no exception will be set when
             attempting to delete a nonexistent image.
@@ -852,7 +848,7 @@ class Proxy(proxy.Proxy):
 
         :param name_or_id: The name or ID of a image.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -871,7 +867,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.image.Image` instance.
 
         :returns: One :class:`~openstack.image.v2.image.Image`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_image.Image, image)
@@ -949,7 +945,7 @@ class Proxy(proxy.Proxy):
         for k, v in iter(kwargs.items()):
             if v and k in ['ramdisk', 'kernel']:
                 v = self._connection.get_image_id(v)
-                k = '{0}_id'.format(k)
+                k = f'{k}_id'
             properties[k] = v
 
         img_props = image.properties.copy()
@@ -1020,7 +1016,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.image.Image` instance that the member
             is part of. This is required if ``member`` is an ID.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the member does not exist. When set to ``True``, no exception will
             be set when attempting to delete a nonexistent member.
 
@@ -1043,7 +1039,7 @@ class Proxy(proxy.Proxy):
             the value can be the ID of a image or a
             :class:`~openstack.image.v2.image.Image` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -1066,7 +1062,7 @@ class Proxy(proxy.Proxy):
             The value can be the ID of a image or a
             :class:`~openstack.image.v2.image.Image` instance.
         :returns: One :class:`~openstack.image.v2.member.Member`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         member_id = resource.Resource._get_id(member)
@@ -1138,7 +1134,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
             instance.
         :param bool ignore_missing: When set to ``False``,
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the metadef namespace does not exist.
         :returns: ``None``
         """
@@ -1161,7 +1157,7 @@ class Proxy(proxy.Proxy):
 
         :returns: One
             :class:`~~openstack.image.v2.metadef_namespace.MetadefNamespace`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource can be found.
         """
         return self._get(
@@ -1174,7 +1170,7 @@ class Proxy(proxy.Proxy):
 
         :returns: A generator object of metadef namespaces
         :rtype: :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._list(_metadef_namespace.MetadefNamespace, **query)
@@ -1236,7 +1232,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
             instance.
         :returns: One :class:`~openstack.image.v2.metadef_object.MetadefObject`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource can be found.
         """
         object_name = resource.Resource._get_id(metadef_object)
@@ -1256,7 +1252,7 @@ class Proxy(proxy.Proxy):
             instance.
 
         :returns: One :class:`~openstack.image.v2.metadef_object.MetadefObject`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource can be found.
         """
         namespace_name = resource.Resource._get_id(namespace)
@@ -1278,10 +1274,11 @@ class Proxy(proxy.Proxy):
             a :class:`~openstack.image.v2.metadef_object.MetadefObject`
 
         :returns: One :class:`~openstack.image.v2.metadef_object.MetadefObject`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource can be found.
         """
         namespace_name = resource.Resource._get_id(namespace)
+        metadef_object = resource.Resource._get_id(metadef_object)
         return self._update(
             _metadef_object.MetadefObject,
             metadef_object,
@@ -1302,7 +1299,7 @@ class Proxy(proxy.Proxy):
             a :class:`~openstack.image.v2.metadef_object.MetadefObject`
 
         :returns: ``None``
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource can be found.
         """
         namespace_name = resource.Resource._get_id(namespace)
@@ -1313,6 +1310,22 @@ class Proxy(proxy.Proxy):
             **attrs,
         )
 
+    def delete_all_metadef_objects(self, namespace):
+        """Delete all objects
+
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :returns: ``None``
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
+            resource can be found.
+        """
+        namespace = self._get_resource(
+            _metadef_namespace.MetadefNamespace, namespace
+        )
+        return namespace.delete_all_objects(self)
+
     # ====== METADEF RESOURCE TYPES ======
     def metadef_resource_types(self, **query):
         """Return a generator of metadef resource types
@@ -1320,7 +1333,7 @@ class Proxy(proxy.Proxy):
         :return: A generator object of metadef resource types
         :rtype:
             :class:`~openstack.image.v2.metadef_resource_type.MetadefResourceType`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._list(_metadef_resource_type.MetadefResourceType, **query)
@@ -1367,7 +1380,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
             instance
         :param bool ignore_missing: When set to ``False``,
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the metadef resource type association does not exist.
         :returns: ``None``
         """
@@ -1389,7 +1402,7 @@ class Proxy(proxy.Proxy):
         :return: A generator object of metadef resource type associations
         :rtype:
             :class:`~openstack.image.v2.metadef_resource_type.MetadefResourceTypeAssociation`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
                 when no resource can be found.
         """
         namespace_name = resource.Resource._get_id(metadef_namespace)
@@ -1462,7 +1475,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
             instance
         :param bool ignore_missing: When set to
-            ``False`` :class:`~openstack.exceptions.ResourceNotFound` will be
+            ``False`` :class:`~openstack.exceptions.NotFoundException` will be
             raised when the instance does not exist. When set to ``True``,
             no exception will be set when attempting to delete a nonexistent
             instance.
@@ -1514,7 +1527,7 @@ class Proxy(proxy.Proxy):
 
         :returns: One
             :class:`~~openstack.image.v2.metadef_property.MetadefProperty`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource can be found.
         """
         namespace_name = resource.Resource._get_id(metadef_namespace)
@@ -1525,12 +1538,29 @@ class Proxy(proxy.Proxy):
             **query,
         )
 
+    def delete_all_metadef_properties(self, metadef_namespace):
+        """Delete all metadata definitions property inside a specific namespace.
+
+        :param metadef_namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+
+        :returns: ``None``
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
+            resource can be found.
+        """
+        namespace = self._get_resource(
+            _metadef_namespace.MetadefNamespace, metadef_namespace
+        )
+        return namespace.delete_all_properties(self)
+
     # ====== SCHEMAS ======
     def get_images_schema(self):
         """Get images schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1543,7 +1573,7 @@ class Proxy(proxy.Proxy):
         """Get single image schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1556,7 +1586,7 @@ class Proxy(proxy.Proxy):
         """Get image members schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1569,7 +1599,7 @@ class Proxy(proxy.Proxy):
         """Get image member schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1582,7 +1612,7 @@ class Proxy(proxy.Proxy):
         """Get image tasks schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1595,7 +1625,7 @@ class Proxy(proxy.Proxy):
         """Get image task schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1608,7 +1638,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition namespace schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1621,7 +1651,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition namespaces schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1634,7 +1664,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition resource type association schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1647,7 +1677,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition resource type associations schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1660,7 +1690,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition object schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1673,7 +1703,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition objects schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1686,7 +1716,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition property schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1699,7 +1729,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition properties schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1712,7 +1742,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition tag schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1725,7 +1755,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition tags schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1753,7 +1783,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.task.Task` instance.
 
         :returns: One :class:`~openstack.image.v2.task.Task`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_task.Task, task)
@@ -1806,10 +1836,8 @@ class Proxy(proxy.Proxy):
         if task.status.lower() == status.lower():
             return task
 
-        name = "{res}:{id}".format(res=task.__class__.__name__, id=task.id)
-        msg = "Timeout waiting for {name} to transition to {status}".format(
-            name=name, status=status
-        )
+        name = f"{task.__class__.__name__}:{task.id}"
+        msg = f"Timeout waiting for {name} to transition to {status}"
 
         for count in utils.iterate_timeout(
             timeout=wait, message=msg, wait=interval
@@ -1818,9 +1846,7 @@ class Proxy(proxy.Proxy):
 
             if not task:
                 raise exceptions.ResourceFailure(
-                    "{name} went away while waiting for {status}".format(
-                        name=name, status=status
-                    )
+                    f"{name} went away while waiting for {status}"
                 )
 
             new_status = task.status
@@ -1831,12 +1857,10 @@ class Proxy(proxy.Proxy):
                 if task.message == _IMAGE_ERROR_396:
                     task_args = {'input': task.input, 'type': task.type}
                     task = self.create_task(**task_args)
-                    self.log.debug('Got error 396. Recreating task %s' % task)
+                    self.log.debug(f'Got error 396. Recreating task {task}')
                 else:
                     raise exceptions.ResourceFailure(
-                        "{name} transitioned to failure state {status}".format(
-                            name=name, status=new_status
-                        )
+                        f"{name} transitioned to failure state {new_status}"
                     )
 
             self.log.debug(
@@ -1863,7 +1887,7 @@ class Proxy(proxy.Proxy):
         """Get a info about image constraints
 
         :returns: One :class:`~openstack.image.v2.service_info.Import`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_si.Import, requires_id=False)
@@ -1883,3 +1907,32 @@ class Proxy(proxy.Proxy):
             to delete failed to occur in the specified seconds.
         """
         return resource.wait_for_delete(self, res, interval, wait)
+
+    def _get_cleanup_dependencies(self):
+        return {'image': {'before': ['identity']}}
+
+    def _service_cleanup(
+        self,
+        dry_run=True,
+        client_status_queue=None,
+        identified_resources=None,
+        filters=None,
+        resource_evaluation_fn=None,
+        skip_resources=None,
+    ):
+        if self.should_skip_resource_cleanup("image", skip_resources):
+            return
+
+        project_id = self.get_project_id()
+
+        # Note that images cannot be deleted when they are still being used
+        for obj in self.images(owner=project_id):
+            self._service_cleanup_del_res(
+                self.delete_image,
+                obj,
+                dry_run=dry_run,
+                client_status_queue=client_status_queue,
+                identified_resources=identified_resources,
+                filters=filters,
+                resource_evaluation_fn=resource_evaluation_fn,
+            )
