@@ -10,13 +10,12 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack import format
+from urllib import parse
 
-from six.moves.urllib import parse
+from openstack import format
 
 
 class HREFToUUID(format.Formatter):
-
     @classmethod
     def deserialize(cls, value):
         """Convert a HREF to the UUID portion"""
@@ -25,15 +24,7 @@ class HREFToUUID(format.Formatter):
         # Only try to proceed if we have an actual URI.
         # Just check that we have a scheme, netloc, and path.
         if not all(parts[:3]):
-            raise ValueError("Unable to convert %s to an ID" % value)
+            raise ValueError(f"Unable to convert {value} to an ID")
 
         # The UUID will be the last portion of the URI.
         return parts.path.split("/")[-1]
-
-    @classmethod
-    def serialize(cls, value):
-        # NOTE(briancurtin): If we had access to the session to get
-        # the endpoint we could do something smart here like take an ID
-        # and give back an HREF, but this will just have to be something
-        # that works different because Barbican does what it does...
-        return value
