@@ -52,7 +52,7 @@ class TestFirewallRule(FirewallTestCase):
     mock_firewall_rule = None
 
     def setUp(self, cloud_config_fixture='clouds.yaml'):
-        super(TestFirewallRule, self).setUp()
+        super().setUp()
         self.mock_firewall_rule = FirewallRule(
             connection=self.cloud, **self._mock_firewall_rule_attrs
         ).to_dict()
@@ -94,7 +94,7 @@ class TestFirewallRule(FirewallTestCase):
         self.assertRaises(
             exceptions.BadRequestException,
             self.cloud.create_firewall_rule,
-            **bad_rule
+            **bad_rule,
         )
         self.assert_calls()
 
@@ -146,7 +146,7 @@ class TestFirewallRule(FirewallTestCase):
                     uri=self._make_mock_url(
                         'firewall_rules',
                         name=self.firewall_rule_name,
-                        **filters
+                        **filters,
                     ),
                     json={'firewall_rules': [self.mock_firewall_rule]},
                 ),
@@ -367,7 +367,7 @@ class TestFirewallPolicy(FirewallTestCase):
     mock_firewall_policy = None
 
     def setUp(self, cloud_config_fixture='clouds.yaml'):
-        super(TestFirewallPolicy, self).setUp()
+        super().setUp()
         self.mock_firewall_policy = FirewallPolicy(
             connection=self.cloud, **self._mock_firewall_policy_attrs
         ).to_dict()
@@ -444,9 +444,9 @@ class TestFirewallPolicy(FirewallTestCase):
 
         with mock.patch.object(self.cloud.network, 'create_firewall_policy'):
             self.assertRaises(
-                exceptions.ResourceNotFound,
+                exceptions.NotFoundException,
                 self.cloud.create_firewall_policy,
-                **posted_policy
+                **posted_policy,
             )
             self.cloud.network.create_firewall_policy.assert_not_called()
             self.assert_calls()
@@ -947,7 +947,7 @@ class TestFirewallPolicy(FirewallTestCase):
 
         with mock.patch.object(self.cloud.network, 'find_firewall_rule'):
             self.assertRaises(
-                exceptions.ResourceNotFound,
+                exceptions.NotFoundException,
                 self.cloud.insert_rule_into_policy,
                 policy_name,
                 'bogus_rule',
@@ -979,7 +979,7 @@ class TestFirewallPolicy(FirewallTestCase):
             ]
         )
         self.assertRaises(
-            exceptions.ResourceNotFound,
+            exceptions.NotFoundException,
             self.cloud.insert_rule_into_policy,
             self.firewall_policy_id,
             rule_name,
@@ -1088,7 +1088,7 @@ class TestFirewallPolicy(FirewallTestCase):
 
         with mock.patch.object(self.cloud.network, 'find_firewall_rule'):
             self.assertRaises(
-                exceptions.ResourceNotFound,
+                exceptions.NotFoundException,
                 self.cloud.remove_rule_from_policy,
                 self.firewall_policy_name,
                 TestFirewallRule.firewall_rule_name,
@@ -1206,7 +1206,7 @@ class TestFirewallGroup(FirewallTestCase):
     mock_returned_firewall_rule = None
 
     def setUp(self, cloud_config_fixture='clouds.yaml'):
-        super(TestFirewallGroup, self).setUp()
+        super().setUp()
         self.mock_egress_policy = FirewallPolicy(
             connection=self.cloud, **self._mock_egress_policy_attrs
         ).to_dict()
@@ -1274,7 +1274,7 @@ class TestFirewallGroup(FirewallTestCase):
                         'network',
                         'public',
                         append=['v2.0', 'ports'],
-                        qs_elements=['name=%s' % self.mock_port['name']],
+                        qs_elements=['name={}'.format(self.mock_port['name'])],
                     ),
                     json={'ports': [self.mock_port]},
                 ),
@@ -1580,7 +1580,7 @@ class TestFirewallGroup(FirewallTestCase):
                         'network',
                         'public',
                         append=['v2.0', 'ports'],
-                        qs_elements=['name=%s' % self.mock_port['name']],
+                        qs_elements=['name={}'.format(self.mock_port['name'])],
                     ),
                     json={'ports': [self.mock_port]},
                 ),
