@@ -44,7 +44,7 @@ class TestIdentityRoles(base.TestCase):
         base_url_append='v3',
         qs_elements=None,
     ):
-        return super(TestIdentityRoles, self).get_mock_url(
+        return super().get_mock_url(
             service_type,
             interface,
             resource,
@@ -75,7 +75,7 @@ class TestIdentityRoles(base.TestCase):
                 dict(
                     method='GET',
                     uri=self.get_mock_url(
-                        qs_elements=['name={0}'.format(role_data.role_name)]
+                        qs_elements=[f'name={role_data.role_name}']
                     ),
                     status_code=200,
                     json={'roles': [role_data.json_response['role']]},
@@ -286,8 +286,8 @@ class TestIdentityRoles(base.TestCase):
                     uri=self.get_mock_url(
                         resource='role_assignments',
                         qs_elements=[
-                            'scope.domain.id=%s' % domain_data.domain_id,
-                            'user.id=%s' % user_data.user_id,
+                            f'scope.domain.id={domain_data.domain_id}',
+                            f'user.id={user_data.user_id}',
                             'effective=True',
                         ],
                     ),
