@@ -13,6 +13,7 @@
 import uuid
 
 from openstack.identity.v3 import _proxy
+from openstack.identity.v3 import access_rule
 from openstack.identity.v3 import credential
 from openstack.identity.v3 import domain
 from openstack.identity.v3 import domain_config
@@ -29,6 +30,7 @@ from openstack.identity.v3 import role_project_user_assignment
 from openstack.identity.v3 import role_system_group_assignment
 from openstack.identity.v3 import role_system_user_assignment
 from openstack.identity.v3 import service
+from openstack.identity.v3 import service_provider
 from openstack.identity.v3 import trust
 from openstack.identity.v3 import user
 from openstack.tests.unit import test_proxy_base
@@ -38,7 +40,7 @@ USER_ID = 'user-id-' + uuid.uuid4().hex
 
 class TestIdentityProxyBase(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestIdentityProxyBase, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
 
@@ -493,6 +495,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(user.User, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -506,6 +509,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(user.User, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -519,6 +523,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(user.User, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -532,6 +537,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(group.Group, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -545,6 +551,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(group.Group, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -558,6 +565,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(group.Group, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -571,6 +579,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(user.User, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -584,6 +593,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(user.User, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -597,6 +607,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(user.User, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -610,6 +621,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(group.Group, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -623,6 +635,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(group.Group, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -636,6 +649,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(group.Group, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -710,3 +724,90 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy._get_resource(role.Role, 'rid'),
             ],
         )
+
+
+class TestAccessRule(TestIdentityProxyBase):
+    def test_access_rule_delete(self):
+        self.verify_delete(
+            self.proxy.delete_access_rule,
+            access_rule.AccessRule,
+            False,
+            method_args=[],
+            method_kwargs={'user': USER_ID, 'access_rule': 'access_rule'},
+            expected_args=['access_rule'],
+            expected_kwargs={'user_id': USER_ID},
+        )
+
+    def test_access_rule_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_access_rule,
+            access_rule.AccessRule,
+            True,
+            method_args=[],
+            method_kwargs={'user': USER_ID, 'access_rule': 'access_rule'},
+            expected_args=['access_rule'],
+            expected_kwargs={'user_id': USER_ID},
+        )
+
+    def test_access_rule_get(self):
+        self.verify_get(
+            self.proxy.get_access_rule,
+            access_rule.AccessRule,
+            method_args=[],
+            method_kwargs={'user': USER_ID, 'access_rule': 'access_rule'},
+            expected_args=['access_rule'],
+            expected_kwargs={'user_id': USER_ID},
+        )
+
+    def test_access_rules(self):
+        self.verify_list(
+            self.proxy.access_rules,
+            access_rule.AccessRule,
+            method_kwargs={'user': USER_ID},
+            expected_kwargs={'user_id': USER_ID},
+        )
+
+
+class TestServiceProvider(TestIdentityProxyBase):
+    def test_service_provider_create(self):
+        self.verify_create(
+            self.proxy.create_service_provider,
+            service_provider.ServiceProvider,
+        )
+
+    def test_service_provider_delete(self):
+        self.verify_delete(
+            self.proxy.delete_service_provider,
+            service_provider.ServiceProvider,
+            False,
+        )
+
+    def test_service_provider_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_service_provider,
+            service_provider.ServiceProvider,
+            True,
+        )
+
+    def test_service_provider_find(self):
+        self.verify_find(
+            self.proxy.find_service_provider, service_provider.ServiceProvider
+        )
+
+    def test_service_provider_get(self):
+        self.verify_get(
+            self.proxy.get_service_provider,
+            service_provider.ServiceProvider,
+        )
+
+    def test_service_providers(self):
+        self.verify_list(
+            self.proxy.service_providers,
+            service_provider.ServiceProvider,
+        )
+
+    def test_service_provider_update(self):
+        self.verify_update(
+            self.proxy.update_service_provider,
+            service_provider.ServiceProvider,
+        )
